import logging

from ClientHolder import ClientHolder, ClientWrapper
from olvid import datatypes, listeners
from olvid.internal import notifications
from utils import send_message_wait_and_check_content


# noinspection PyProtectedMember,DuplicatedCode
async def first_routine(client_1: ClientWrapper, client_2: ClientWrapper):
	# send a message to react to
	sent_message, received_message = await send_message_wait_and_check_content(client_1, client_2, "Message to react to")

	####
	# Add reaction
	####
	ideal_reaction_added_notif_1 = notifications.MessageReactionAddedNotification(message=sent_message._clone(), reaction=datatypes.MessageReaction(contact_id=0, reaction="A"))
	ideal_reaction_added_notif_1.message.reactions = [ideal_reaction_added_notif_1.reaction]
	ideal_reaction_added_notif_2 = notifications.MessageReactionAddedNotification(message=received_message._clone(), reaction=datatypes.MessageReaction(contact_id=received_message.sender_id, reaction="A"))
	ideal_reaction_added_notif_2.message.reactions = [ideal_reaction_added_notif_2.reaction]
	bot_1 = client_1.create_notification_bot(listeners.GenericNotificationListener(notification_type=listeners.NOTIFICATIONS.MESSAGE_REACTION_ADDED, handler=client_1.get_check_content_handler(ideal_reaction_added_notif_1), count=1))
	bot_2 = client_2.create_notification_bot(listeners.GenericNotificationListener(notification_type=listeners.NOTIFICATIONS.MESSAGE_REACTION_ADDED, handler=client_2.get_check_content_handler(ideal_reaction_added_notif_2), count=1))

	await client_1.message_react(sent_message.id, "A")
	await bot_1.wait_for_listeners_end()
	await bot_2.wait_for_listeners_end()

	####
	# Update reaction (1)
	####
	ideal_reaction_updated_notif_1 = notifications.MessageReactionUpdatedNotification(message=sent_message._clone(), reaction=datatypes.MessageReaction(contact_id=0, reaction="B"), previous_reaction=datatypes.MessageReaction(contact_id=0, reaction="A"))
	ideal_reaction_updated_notif_1.message.reactions = [ideal_reaction_updated_notif_1.reaction]
	ideal_reaction_updated_notif_2 = notifications.MessageReactionUpdatedNotification(message=received_message._clone(), reaction=datatypes.MessageReaction(contact_id=0, reaction="B"), previous_reaction=datatypes.MessageReaction(contact_id=0, reaction="A"))
	ideal_reaction_updated_notif_2.message.reactions = [ideal_reaction_updated_notif_2.reaction]
	bot_1 = client_1.create_notification_bot(listeners.GenericNotificationListener(notification_type=listeners.NOTIFICATIONS.MESSAGE_REACTION_UPDATED, handler=client_1.get_check_content_handler(ideal_reaction_updated_notif_1), count=1))
	bot_2 = client_2.create_notification_bot(listeners.GenericNotificationListener(notification_type=listeners.NOTIFICATIONS.MESSAGE_REACTION_UPDATED, handler=client_2.get_check_content_handler(ideal_reaction_updated_notif_2), count=1))

	await client_1.message_react(sent_message.id, "B")
	await bot_1.wait_for_listeners_end()
	await bot_2.wait_for_listeners_end()

	####
	# Update reaction (2)
	####
	ideal_reaction_updated_notif_1 = notifications.MessageReactionUpdatedNotification(message=sent_message._clone(), reaction=datatypes.MessageReaction(contact_id=0, reaction="C"), previous_reaction=datatypes.MessageReaction(contact_id=0, reaction="B"))
	ideal_reaction_updated_notif_1.message.reactions = [ideal_reaction_updated_notif_1.reaction]
	ideal_reaction_updated_notif_2 = notifications.MessageReactionUpdatedNotification(message=received_message._clone(), reaction=datatypes.MessageReaction(contact_id=0, reaction="C"), previous_reaction=datatypes.MessageReaction(contact_id=0, reaction="B"))
	ideal_reaction_updated_notif_2.message.reactions = [ideal_reaction_updated_notif_2.reaction]
	bot_1 = client_1.create_notification_bot(listeners.GenericNotificationListener(notification_type=listeners.NOTIFICATIONS.MESSAGE_REACTION_UPDATED, handler=client_1.get_check_content_handler(ideal_reaction_updated_notif_1), count=1))
	bot_2 = client_2.create_notification_bot(listeners.GenericNotificationListener(notification_type=listeners.NOTIFICATIONS.MESSAGE_REACTION_UPDATED, handler=client_2.get_check_content_handler(ideal_reaction_updated_notif_2), count=1))

	await client_1.message_react(sent_message.id, "C")
	await bot_1.wait_for_listeners_end()
	await bot_2.wait_for_listeners_end()

	####
	# Delete reaction
	####
	ideal_reaction_removed_notif_1 = notifications.MessageReactionRemovedNotification(message=sent_message._clone(), reaction=datatypes.MessageReaction(contact_id=0, reaction="C"))
	ideal_reaction_removed_notif_2 = notifications.MessageReactionRemovedNotification(message=received_message._clone(), reaction=datatypes.MessageReaction(contact_id=received_message.sender_id, reaction="C"))
	bot_1 = client_1.create_notification_bot(listeners.GenericNotificationListener(notification_type=listeners.NOTIFICATIONS.MESSAGE_REACTION_REMOVED, handler=client_1.get_check_content_handler(ideal_reaction_removed_notif_1), count=1))
	bot_2 = client_2.create_notification_bot(listeners.GenericNotificationListener(notification_type=listeners.NOTIFICATIONS.MESSAGE_REACTION_REMOVED, handler=client_2.get_check_content_handler(ideal_reaction_removed_notif_2), count=1))

	await client_1.message_react(sent_message.id, reaction="")

	await bot_1.wait_for_listeners_end()
	await bot_2.wait_for_listeners_end()


# noinspection PyUnusedLocal
async def test_reaction(client_holder: ClientHolder, fast_mode):
	for c1, c2 in client_holder.get_all_client_pairs():
		logging.info(f"Reaction routine: {c1.identity.id} <-> {c2.identity.id}")
		await first_routine(c1, c2)



import asyncclick as click

from .ClientSingleton import ClientSingleton
from typing import Any

from olvid import datatypes

from ..tools.exceptions import CancelCommandError


def _recursive_get_attr(item: object, attributes: list[str]):
	if not attributes:
		return item
	return _recursive_get_attr(getattr(item, attributes[0]), attributes[1:])


def filter_fields_and_print_normal_message(item: object, fields: str):
	if not fields:
		print_normal_message(item, item)
	else:
		try:
			field_names = fields.split(",")
			if len(field_names) == 1:
				attr = _recursive_get_attr(item, field_names[0].split('.'))
				if type(attr) in [tuple, list]:
					attr = ",".join([f"({a})" for a in attr])
				normal_str: str = f"{attr}"
				script_str: str = f"{attr}"
			else:
				normal_str: str = ""
				script_str: str = ""
				for field_name in field_names:
					attr = _recursive_get_attr(item, field_name.split('.'))
					if type(attr) in [tuple, list]:
						attr = ",".join([f"({a})" for a in attr])
					script_str += f"{attr},"
					normal_str += f"{field_name}: {attr},"
				normal_str = normal_str.strip().removesuffix(",")
				script_str = script_str.strip().removesuffix(",")
			print_normal_message(normal_str, script_str)
		except AttributeError as e:
			print_error_message(f"Invalid fields filter: {fields}")
			print_error_message(e)
			raise CancelCommandError()


# print a red message on stderr
def print_error_message(message: Any):
	click.secho("{}".format(message), fg="red", err=True)


# print a yellow message on stderr
def print_warning_message(message: Any):
	click.secho("{}".format(message), fg="yellow", err=True)


# print a message if not in script mode
def print_command_result(message: Any, script_message: Any = None):
	if not ClientSingleton.is_script_mode_enabled():
		click.secho("{}".format(message), fg="cyan")
	elif script_message:
		click.secho(str(script_message))


def print_debug_message(message: Any, script_version: Any):
	if ClientSingleton.is_script_mode_enabled():
		if script_version:
			click.secho("{}".format(script_version), fg="magenta", err=True)
	else:
		click.secho("{}".format(message), fg="magenta", err=True)


def print_success_message(message: Any, script_version: Any):
	if ClientSingleton.is_script_mode_enabled():
		if script_version:
			click.secho("{}".format(script_version), fg="green")
	else:
		click.secho("{}".format(message), fg="green")


def print_normal_message(message: Any, script_version: Any):
	if ClientSingleton.is_script_mode_enabled():
		if script_version:
			click.secho("{}".format(script_version))
	else:
		click.secho("{}".format(message))


def string_to_message_id(message_id: str) -> datatypes.MessageId:
	if len(message_id) <= 1:
		raise click.exceptions.NoSuchOption(f"Invalid message id: {message_id}")
	try:
		int_id = int(message_id[1:])
	except ValueError:
		raise click.exceptions.NoSuchOption(f"Invalid message id: {message_id}")

	if message_id[0].upper() == "I":
		return datatypes.MessageId(type=datatypes.MessageId.Type.TYPE_INBOUND, id=int_id)
	elif message_id[0].upper() == "O":
		return datatypes.MessageId(type=datatypes.MessageId.Type.TYPE_OUTBOUND, id=int_id)
	raise click.exceptions.NoSuchOption(f"Invalid message id: {message_id}")


def string_to_attachment_id(attachment_id: str) -> datatypes.AttachmentId:
	if len(attachment_id) <= 1:
		raise click.exceptions.NoSuchOption(f"Invalid attachment id: {attachment_id}")
	try:
		int_id = int(attachment_id[1:])
	except ValueError:
		raise click.exceptions.NoSuchOption(f"Invalid attachment id: {attachment_id}")

	if attachment_id[0].upper() == "I":
		return datatypes.AttachmentId(type=datatypes.AttachmentId.Type.TYPE_INBOUND, id=int_id)
	elif attachment_id[0].upper() == "O":
		return datatypes.AttachmentId(type=datatypes.AttachmentId.Type.TYPE_OUTBOUND, id=int_id)
	raise click.exceptions.NoSuchOption(f"Invalid attachment id: {attachment_id}")


def message_id_to_string(message_id: datatypes.MessageId):
	return f"{message_id.type.name[0]}{message_id.id}"


def attachment_id_to_string(attachment_id: datatypes.AttachmentId):
	return f"{attachment_id.type.name[0]}{attachment_id.id}"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: olvid/daemon/services/v1/command_service.proto
# Protobuf Python Version: 5.27.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    3,
    '',
    'olvid/daemon/services/v1/command_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .....olvid.daemon.command.v1 import attachment_commands_pb2 as olvid_dot_daemon_dot_command_dot_v1_dot_attachment__commands__pb2
from .....olvid.daemon.command.v1 import contact_commands_pb2 as olvid_dot_daemon_dot_command_dot_v1_dot_contact__commands__pb2
from .....olvid.daemon.command.v1 import discussion_commands_pb2 as olvid_dot_daemon_dot_command_dot_v1_dot_discussion__commands__pb2
from .....olvid.daemon.command.v1 import group_commands_pb2 as olvid_dot_daemon_dot_command_dot_v1_dot_group__commands__pb2
from .....olvid.daemon.command.v1 import identity_commands_pb2 as olvid_dot_daemon_dot_command_dot_v1_dot_identity__commands__pb2
from .....olvid.daemon.command.v1 import invitation_commands_pb2 as olvid_dot_daemon_dot_command_dot_v1_dot_invitation__commands__pb2
from .....olvid.daemon.command.v1 import message_commands_pb2 as olvid_dot_daemon_dot_command_dot_v1_dot_message__commands__pb2
from .....olvid.daemon.command.v1 import storage_commands_pb2 as olvid_dot_daemon_dot_command_dot_v1_dot_storage__commands__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.olvid/daemon/services/v1/command_service.proto\x12\x18olvid.daemon.services.v1\x1a\x31olvid/daemon/command/v1/attachment_commands.proto\x1a.olvid/daemon/command/v1/contact_commands.proto\x1a\x31olvid/daemon/command/v1/discussion_commands.proto\x1a,olvid/daemon/command/v1/group_commands.proto\x1a/olvid/daemon/command/v1/identity_commands.proto\x1a\x31olvid/daemon/command/v1/invitation_commands.proto\x1a.olvid/daemon/command/v1/message_commands.proto\x1a.olvid/daemon/command/v1/storage_commands.proto2\xc5\x08\n\x16IdentityCommandService\x12j\n\x0bIdentityGet\x12+.olvid.daemon.command.v1.IdentityGetRequest\x1a,.olvid.daemon.command.v1.IdentityGetResponse\"\x00\x12\x88\x01\n\x15IdentityUpdateDetails\x12\x35.olvid.daemon.command.v1.IdentityUpdateDetailsRequest\x1a\x36.olvid.daemon.command.v1.IdentityUpdateDetailsResponse\"\x00\x12\x82\x01\n\x13IdentityRemovePhoto\x12\x33.olvid.daemon.command.v1.IdentityRemovePhotoRequest\x1a\x34.olvid.daemon.command.v1.IdentityRemovePhotoResponse\"\x00\x12{\n\x10IdentitySetPhoto\x12\x30.olvid.daemon.command.v1.IdentitySetPhotoRequest\x1a\x31.olvid.daemon.command.v1.IdentitySetPhotoResponse\"\x00(\x01\x12\x85\x01\n\x14IdentityKeycloakBind\x12\x34.olvid.daemon.command.v1.IdentityKeycloakBindRequest\x1a\x35.olvid.daemon.command.v1.IdentityKeycloakBindResponse\"\x00\x12\x8b\x01\n\x16IdentityKeycloakUnbind\x12\x36.olvid.daemon.command.v1.IdentityKeycloakUnbindRequest\x1a\x37.olvid.daemon.command.v1.IdentityKeycloakUnbindResponse\"\x00\x12|\n\x11IdentitySetApiKey\x12\x31.olvid.daemon.command.v1.IdentitySetApiKeyRequest\x1a\x32.olvid.daemon.command.v1.IdentitySetApiKeyResponse\"\x00\x12\x9d\x01\n\x1cIdentitySetConfigurationLink\x12<.olvid.daemon.command.v1.IdentitySetConfigurationLinkRequest\x1a=.olvid.daemon.command.v1.IdentitySetConfigurationLinkResponse\"\x00\x32\xdb\x06\n\x18InvitationCommandService\x12u\n\x0eInvitationList\x12..olvid.daemon.command.v1.InvitationListRequest\x1a/.olvid.daemon.command.v1.InvitationListResponse\"\x00\x30\x01\x12p\n\rInvitationGet\x12-.olvid.daemon.command.v1.InvitationGetRequest\x1a..olvid.daemon.command.v1.InvitationGetResponse\"\x00\x12p\n\rInvitationNew\x12-.olvid.daemon.command.v1.InvitationNewRequest\x1a..olvid.daemon.command.v1.InvitationNewResponse\"\x00\x12y\n\x10InvitationAccept\x12\x30.olvid.daemon.command.v1.InvitationAcceptRequest\x1a\x31.olvid.daemon.command.v1.InvitationAcceptResponse\"\x00\x12|\n\x11InvitationDecline\x12\x31.olvid.daemon.command.v1.InvitationDeclineRequest\x1a\x32.olvid.daemon.command.v1.InvitationDeclineResponse\"\x00\x12p\n\rInvitationSas\x12-.olvid.daemon.command.v1.InvitationSasRequest\x1a..olvid.daemon.command.v1.InvitationSasResponse\"\x00\x12y\n\x10InvitationDelete\x12\x30.olvid.daemon.command.v1.InvitationDeleteRequest\x1a\x31.olvid.daemon.command.v1.InvitationDeleteResponse\"\x00\x32\xc6\x06\n\x15\x43ontactCommandService\x12l\n\x0b\x43ontactList\x12+.olvid.daemon.command.v1.ContactListRequest\x1a,.olvid.daemon.command.v1.ContactListResponse\"\x00\x30\x01\x12g\n\nContactGet\x12*.olvid.daemon.command.v1.ContactGetRequest\x1a+.olvid.daemon.command.v1.ContactGetResponse\"\x00\x12p\n\rContactDelete\x12-.olvid.daemon.command.v1.ContactDeleteRequest\x1a..olvid.daemon.command.v1.ContactDeleteResponse\"\x00\x12\x82\x01\n\x13\x43ontactIntroduction\x12\x33.olvid.daemon.command.v1.ContactIntroductionRequest\x1a\x34.olvid.daemon.command.v1.ContactIntroductionResponse\"\x00\x12\xac\x01\n!ContactInviteToOneToOneDiscussion\x12\x41.olvid.daemon.command.v1.ContactInviteToOneToOneDiscussionRequest\x1a\x42.olvid.daemon.command.v1.ContactInviteToOneToOneDiscussionResponse\"\x00\x12\xaf\x01\n\"ContactDowngradeOneToOneDiscussion\x12\x42.olvid.daemon.command.v1.ContactDowngradeOneToOneDiscussionRequest\x1a\x43.olvid.daemon.command.v1.ContactDowngradeOneToOneDiscussionResponse\"\x00\x32\xc2\n\n\x13GroupCommandService\x12\x66\n\tGroupList\x12).olvid.daemon.command.v1.GroupListRequest\x1a*.olvid.daemon.command.v1.GroupListResponse\"\x00\x30\x01\x12\x61\n\x08GroupGet\x12(.olvid.daemon.command.v1.GroupGetRequest\x1a).olvid.daemon.command.v1.GroupGetResponse\"\x00\x12\x88\x01\n\x15GroupNewStandardGroup\x12\x35.olvid.daemon.command.v1.GroupNewStandardGroupRequest\x1a\x36.olvid.daemon.command.v1.GroupNewStandardGroupResponse\"\x00\x12\x8e\x01\n\x17GroupNewControlledGroup\x12\x37.olvid.daemon.command.v1.GroupNewControlledGroupRequest\x1a\x38.olvid.daemon.command.v1.GroupNewControlledGroupResponse\"\x00\x12\x88\x01\n\x15GroupNewReadOnlyGroup\x12\x35.olvid.daemon.command.v1.GroupNewReadOnlyGroupRequest\x1a\x36.olvid.daemon.command.v1.GroupNewReadOnlyGroupResponse\"\x00\x12\x88\x01\n\x15GroupNewAdvancedGroup\x12\x35.olvid.daemon.command.v1.GroupNewAdvancedGroupRequest\x1a\x36.olvid.daemon.command.v1.GroupNewAdvancedGroupResponse\"\x00\x12m\n\x0cGroupDisband\x12,.olvid.daemon.command.v1.GroupDisbandRequest\x1a-.olvid.daemon.command.v1.GroupDisbandResponse\"\x00\x12g\n\nGroupLeave\x12*.olvid.daemon.command.v1.GroupLeaveRequest\x1a+.olvid.daemon.command.v1.GroupLeaveResponse\"\x00\x12j\n\x0bGroupUpdate\x12+.olvid.daemon.command.v1.GroupUpdateRequest\x1a,.olvid.daemon.command.v1.GroupUpdateResponse\"\x00\x12v\n\x0fGroupUnsetPhoto\x12/.olvid.daemon.command.v1.GroupUnsetPhotoRequest\x1a\x30.olvid.daemon.command.v1.GroupUnsetPhotoResponse\"\x00\x12r\n\rGroupSetPhoto\x12-.olvid.daemon.command.v1.GroupSetPhotoRequest\x1a..olvid.daemon.command.v1.GroupSetPhotoResponse\"\x00(\x01\x32\xbf\t\n\x18\x44iscussionCommandService\x12u\n\x0e\x44iscussionList\x12..olvid.daemon.command.v1.DiscussionListRequest\x1a/.olvid.daemon.command.v1.DiscussionListResponse\"\x00\x30\x01\x12p\n\rDiscussionGet\x12-.olvid.daemon.command.v1.DiscussionGetRequest\x1a..olvid.daemon.command.v1.DiscussionGetResponse\"\x00\x12\x8b\x01\n\x16\x44iscussionGetByContact\x12\x36.olvid.daemon.command.v1.DiscussionGetByContactRequest\x1a\x37.olvid.daemon.command.v1.DiscussionGetByContactResponse\"\x00\x12\x85\x01\n\x14\x44iscussionGetByGroup\x12\x34.olvid.daemon.command.v1.DiscussionGetByGroupRequest\x1a\x35.olvid.daemon.command.v1.DiscussionGetByGroupResponse\"\x00\x12v\n\x0f\x44iscussionEmpty\x12/.olvid.daemon.command.v1.DiscussionEmptyRequest\x1a\x30.olvid.daemon.command.v1.DiscussionEmptyResponse\"\x00\x12\x88\x01\n\x15\x44iscussionSettingsGet\x12\x35.olvid.daemon.command.v1.DiscussionSettingsGetRequest\x1a\x36.olvid.daemon.command.v1.DiscussionSettingsGetResponse\"\x00\x12\x88\x01\n\x15\x44iscussionSettingsSet\x12\x35.olvid.daemon.command.v1.DiscussionSettingsSetRequest\x1a\x36.olvid.daemon.command.v1.DiscussionSettingsSetResponse\"\x00\x12\x87\x01\n\x14\x44iscussionLockedList\x12\x34.olvid.daemon.command.v1.DiscussionLockedListRequest\x1a\x35.olvid.daemon.command.v1.DiscussionLockedListResponse\"\x00\x30\x01\x12\x8b\x01\n\x16\x44iscussionLockedDelete\x12\x36.olvid.daemon.command.v1.DiscussionLockedDeleteRequest\x1a\x37.olvid.daemon.command.v1.DiscussionLockedDeleteResponse\"\x00\x32\xc2\x08\n\x15MessageCommandService\x12l\n\x0bMessageList\x12+.olvid.daemon.command.v1.MessageListRequest\x1a,.olvid.daemon.command.v1.MessageListResponse\"\x00\x30\x01\x12g\n\nMessageGet\x12*.olvid.daemon.command.v1.MessageGetRequest\x1a+.olvid.daemon.command.v1.MessageGetResponse\"\x00\x12s\n\x0eMessageRefresh\x12..olvid.daemon.command.v1.MessageRefreshRequest\x1a/.olvid.daemon.command.v1.MessageRefreshResponse\"\x00\x12p\n\rMessageDelete\x12-.olvid.daemon.command.v1.MessageDeleteRequest\x1a..olvid.daemon.command.v1.MessageDeleteResponse\"\x00\x12j\n\x0bMessageSend\x12+.olvid.daemon.command.v1.MessageSendRequest\x1a,.olvid.daemon.command.v1.MessageSendResponse\"\x00\x12\x99\x01\n\x1aMessageSendWithAttachments\x12:.olvid.daemon.command.v1.MessageSendWithAttachmentsRequest\x1a;.olvid.daemon.command.v1.MessageSendWithAttachmentsResponse\"\x00(\x01\x12m\n\x0cMessageReact\x12,.olvid.daemon.command.v1.MessageReactRequest\x1a-.olvid.daemon.command.v1.MessageReactResponse\"\x00\x12|\n\x11MessageUpdateBody\x12\x31.olvid.daemon.command.v1.MessageUpdateBodyRequest\x1a\x32.olvid.daemon.command.v1.MessageUpdateBodyResponse\"\x00\x12v\n\x0fMessageSendVoip\x12/.olvid.daemon.command.v1.MessageSendVoipRequest\x1a\x30.olvid.daemon.command.v1.MessageSendVoipResponse\"\x00\x32\x82\x04\n\x18\x41ttachmentCommandService\x12u\n\x0e\x41ttachmentList\x12..olvid.daemon.command.v1.AttachmentListRequest\x1a/.olvid.daemon.command.v1.AttachmentListResponse\"\x00\x30\x01\x12p\n\rAttachmentGet\x12-.olvid.daemon.command.v1.AttachmentGetRequest\x1a..olvid.daemon.command.v1.AttachmentGetResponse\"\x00\x12y\n\x10\x41ttachmentDelete\x12\x30.olvid.daemon.command.v1.AttachmentDeleteRequest\x1a\x31.olvid.daemon.command.v1.AttachmentDeleteResponse\"\x00\x12\x81\x01\n\x12\x41ttachmentDownload\x12\x32.olvid.daemon.command.v1.AttachmentDownloadRequest\x1a\x33.olvid.daemon.command.v1.AttachmentDownloadResponse\"\x00\x30\x01\x32\xc6\x03\n\x15StorageCommandService\x12l\n\x0bStorageList\x12+.olvid.daemon.command.v1.StorageListRequest\x1a,.olvid.daemon.command.v1.StorageListResponse\"\x00\x30\x01\x12g\n\nStorageGet\x12*.olvid.daemon.command.v1.StorageGetRequest\x1a+.olvid.daemon.command.v1.StorageGetResponse\"\x00\x12g\n\nStorageSet\x12*.olvid.daemon.command.v1.StorageSetRequest\x1a+.olvid.daemon.command.v1.StorageSetResponse\"\x00\x12m\n\x0cStorageUnset\x12,.olvid.daemon.command.v1.StorageUnsetRequest\x1a-.olvid.daemon.command.v1.StorageUnsetResponse\"\x00\x32\xcc\x04\n\x1f\x44iscussionStorageCommandService\x12\x8a\x01\n\x15\x44iscussionStorageList\x12\x35.olvid.daemon.command.v1.DiscussionStorageListRequest\x1a\x36.olvid.daemon.command.v1.DiscussionStorageListResponse\"\x00\x30\x01\x12\x85\x01\n\x14\x44iscussionStorageGet\x12\x34.olvid.daemon.command.v1.DiscussionStorageGetRequest\x1a\x35.olvid.daemon.command.v1.DiscussionStorageGetResponse\"\x00\x12\x85\x01\n\x14\x44iscussionStorageSet\x12\x34.olvid.daemon.command.v1.DiscussionStorageSetRequest\x1a\x35.olvid.daemon.command.v1.DiscussionStorageSetResponse\"\x00\x12\x8b\x01\n\x16\x44iscussionStorageUnset\x12\x36.olvid.daemon.command.v1.DiscussionStorageUnsetRequest\x1a\x37.olvid.daemon.command.v1.DiscussionStorageUnsetResponse\"\x00\x42\xc7\x01\n\x1c\x63om.olvid.daemon.services.v1B\x13\x43ommandServiceProtoP\x01Z\x0folvid.io/daemon\xa2\x02\x03ODS\xaa\x02\x18Olvid.Daemon.Services.V1\xca\x02\x18Olvid\\Daemon\\Services\\V1\xe2\x02$Olvid\\Daemon\\Services\\V1\\GPBMetadata\xea\x02\x1bOlvid::Daemon::Services::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'olvid.daemon.services.v1.command_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.olvid.daemon.services.v1B\023CommandServiceProtoP\001Z\017olvid.io/daemon\242\002\003ODS\252\002\030Olvid.Daemon.Services.V1\312\002\030Olvid\\Daemon\\Services\\V1\342\002$Olvid\\Daemon\\Services\\V1\\GPBMetadata\352\002\033Olvid::Daemon::Services::V1'
  _globals['_IDENTITYCOMMANDSERVICE']._serialized_start=469
  _globals['_IDENTITYCOMMANDSERVICE']._serialized_end=1562
  _globals['_INVITATIONCOMMANDSERVICE']._serialized_start=1565
  _globals['_INVITATIONCOMMANDSERVICE']._serialized_end=2424
  _globals['_CONTACTCOMMANDSERVICE']._serialized_start=2427
  _globals['_CONTACTCOMMANDSERVICE']._serialized_end=3265
  _globals['_GROUPCOMMANDSERVICE']._serialized_start=3268
  _globals['_GROUPCOMMANDSERVICE']._serialized_end=4614
  _globals['_DISCUSSIONCOMMANDSERVICE']._serialized_start=4617
  _globals['_DISCUSSIONCOMMANDSERVICE']._serialized_end=5832
  _globals['_MESSAGECOMMANDSERVICE']._serialized_start=5835
  _globals['_MESSAGECOMMANDSERVICE']._serialized_end=6925
  _globals['_ATTACHMENTCOMMANDSERVICE']._serialized_start=6928
  _globals['_ATTACHMENTCOMMANDSERVICE']._serialized_end=7442
  _globals['_STORAGECOMMANDSERVICE']._serialized_start=7445
  _globals['_STORAGECOMMANDSERVICE']._serialized_end=7899
  _globals['_DISCUSSIONSTORAGECOMMANDSERVICE']._serialized_start=7902
  _globals['_DISCUSSIONSTORAGECOMMANDSERVICE']._serialized_end=8490
# @@protoc_insertion_point(module_scope)



from __future__ import annotations
from typing import TYPE_CHECKING

if TYPE_CHECKING:
	from olvid.listeners.GenericNotificationListener import GenericNotificationListener
	from typing import Optional, Coroutine, AsyncIterator

	# for compatibility with python 3.10
	from typing import TypeVar
	# noinspection PyTypeHints
	Self = TypeVar("OlvidClient")

import grpc
import asyncio
import signal
import os
from asyncio import Task

from .logger import core_logger, command_logger, notification_logger
from ..internal import commands, notifications
from .. import datatypes
from ..listeners.ClientListenerHolder import ClientListenerHolder
from ..listeners.Command import Command
from .StubHolder import StubHolder


# noinspection PyProtectedMember,PyShadowingBuiltins
class OlvidClient:
	"""
	OlvidClient: basic class to interact with Olvid daemon.

	OlvidClient needs a client key to authenticate on daemon, you can pass it using:
	- by setting OLVID_CLIENT_KEY env variable
	- by writing it in a .client_key file
	- by passing it as a client_key constructor parameter (to avoid)

	By default, client connects to "localhost:50051" you can change this behavior:
	- set DAEMON_HOSTNAME and/or DAEMON_PORT env variable
	- use server_target parameter. It must be the full server address including hostname/ip and port

	OlvidClient implements every gRPC command methods defined in daemon API.
	You can find methods using the same name as in gRPC but using snake case.
	Request and Response encapsulation layer is fully invisible, you won't need to use Request and Response messages.
	For example for message_send method you can use:
	`client.message_send(discussion_id, body)` and it will return a `datatypes.Message` item.
	you don't use MessageSendRequest and MessageSendResponse.

	OlvidClient also implements a listener mechanism to listen to notification implemented in grpc Notification services.
	Use this code to add a listener to message_received notification:
	`client.add_listener(listeners.MessageReceivedListener(handler=lambda message: print(message)))`
	Again you won't need to use encapsulation messages SubscribeToMessageSendNotification and MessageReceivedNotification.
	"""
	_CHUNK_LENGTH = 1_000_000

	_KEY_ENV_VARIABLE_NAME: str = "OLVID_CLIENT_KEY"
	_KEY_FILE_PATH: str = ".client_key"

	_HOSTNAME_DEFAULT_VALUE: str = "localhost"
	_HOSTNAME_ENV_VARIABLE_NAME: str = "DAEMON_HOSTNAME"

	_PORT_DEFAULT_VALUE: str = "50051"
	_PORT_ENV_VARIABLE_NAME: str = "DAEMON_PORT"

	# we store running clients to notify them if we receive a stop signal
	_running_clients: list[Self] = []

	def __init__(self, client_key: Optional[str] = None, server_target: Optional[str] = None, parent_client: Optional[Self] = None):
		self._stopped = False

		# determine the client key to use (argument > parent > env > file)
		if client_key:
			self._client_key: str = client_key
		elif parent_client:
			self._client_key = parent_client.client_key
		elif os.environ.get(self._KEY_ENV_VARIABLE_NAME):
			self._client_key = os.environ.get(self._KEY_ENV_VARIABLE_NAME).strip()
		elif os.path.isfile(self._KEY_FILE_PATH):
			self._client_key = open(self._KEY_FILE_PATH).read().strip()
		else:
			raise ValueError("Client key not found")

		# determine target (argument > parent > env > default)
		if server_target:
			self._server_target = server_target
		elif parent_client:
			self._server_target = parent_client.server_target
		else:
			self._hostname = os.environ.get(self._HOSTNAME_ENV_VARIABLE_NAME).strip() if os.environ.get(self._HOSTNAME_ENV_VARIABLE_NAME) else self._HOSTNAME_DEFAULT_VALUE
			self._port = os.environ.get(self._PORT_ENV_VARIABLE_NAME).strip() if os.environ.get(self._PORT_ENV_VARIABLE_NAME) else self._PORT_DEFAULT_VALUE
			self._server_target = f"{self._hostname}:{self._port}"

		# store parent client
		self._parent_client: Optional[Self] = parent_client

		# children client case
		if self._parent_client is not None:
			# check parent is running
			if self._parent_client and self._parent_client._stopped:
				raise RuntimeError(f"{self.__class__.__name__}: parent client have been stopped")
			# register as parent's children
			self._parent_client._register_child(self)
			# re-use parent channel
			self._channel: grpc.Channel = self._parent_client._channel
		else:
			self._channel: grpc.Channel = grpc.aio.insecure_channel(target=self._server_target)
		self._registered_child: Optional[list[Self]] = []

		# create or re-use grpc stubs
		self._stubs = StubHolder(client=self, channel=self._channel, parent=self._parent_client)

		# every client keep a list of own registered listeners because listener holder might contain other listeners from parent / son clients
		self._listeners_set: set[GenericNotificationListener] = set()
		# initialize listener holder: warning: this will set up an asyncio loop and cause issues with asyncio.run method that will create its own event loop
		self._listener_holder = ClientListenerHolder(self) if self._parent_client is None else self._parent_client._listener_holder

		# keep a set with pending tasks to keep a strong reference on it (each task might remove itself from the task set when finished)
		self.__task_set = set()

		# register this client to stop it if a SIGTERM signal is received (signal sent when you docker container stops)
		if len(self._running_clients) == 0:
			# if this is the first running client, register the signal handler
			for s in (signal.SIGTERM, ):
				asyncio.get_event_loop().add_signal_handler(s, self.__stop_signal_handler)
		self._running_clients.append(self)

	async def stop(self):
		if self._stopped:
			core_logger.warning(f"{self.__class__.__name__}: trying to stop a stopped instance")
			return

		# stop children
		for child in self._registered_child:
			if not child._stopped:
				await child.stop()

		# remove listeners from holder
		self.remove_all_listeners()

		# if no parent close channels and stop listener_holder
		if self._parent_client is None:
			if self._listener_holder:
				await self._listener_holder.stop()
			if self._channel:
				await self._channel.close()

		self._listener_holder = None

		self._running_clients.remove(self)

		self._stopped = True

		core_logger.debug(f"{self.__class__.__name__}: stopped")

	def _register_child(self, child_client: OlvidClient):
		self._registered_child.append(child_client)

	def are_listeners_finished(self) -> bool:
		self._listeners_set = set([listener for listener in self._listeners_set if not listener.is_finished])
		return len(self._listeners_set) == 0

	async def wait_for_listeners_end(self):
		# if bot was stopped or is not running, there is nothing to wait
		if self._stopped:
			return
		# wait for listeners' end
		while not self.are_listeners_finished():
			await self._listener_holder.wait_for_listener_removed_event()

	#####
	# read only properties
	#####
	@property
	def client_key(self) -> str:
		return self._client_key

	@property
	def server_target(self) -> str:
		return self._server_target

	@staticmethod
	def __stop_signal_handler():
		core_logger.info("Received a stop signal, stopping every running clients")
		for client in OlvidClient._running_clients:
			client.add_background_task(client.stop(), f"force-stop-client")

	#####
	# Background tasks api
	#####
	# this api keeps a reference on created task for you (necessary when running an async task)
	def add_background_task(self, coroutine: Coroutine, name: str = "") -> Task:
		task = asyncio.get_event_loop().create_task(coroutine, name=name if name else None)
		self.__task_set.add(task)

		def end_callback(t):
			self.__task_set.remove(t)
		task.add_done_callback(end_callback)
		return task

	#####
	# Listeners api
	####
	def add_listener(self, listener: GenericNotificationListener):
		if self._stopped:
			raise RuntimeError("Cannot add a listener to a stopped instance")

		if listener not in self._listeners_set:
			self._listeners_set.add(listener)
		self._listener_holder.add_listener(listener)

		# log
		if isinstance(listener, Command):
			core_logger.debug(f"{self.__class__.__name__}: command added: {listener}")
		else:
			core_logger.debug(f"{self.__class__.__name__}: listener added: {listener}")

	def remove_listener(self, listener: GenericNotificationListener):
		if self._stopped:
			core_logger.warning(f"{self.__class__.__name__}: removing a listener on a stopped instance")

		self._listeners_set.discard(listener)
		self._listener_holder.remove_listener(listener)

		# log
		if isinstance(listener, Command):
			core_logger.debug(f"{self.__class__.__name__}: command removed: {listener}")
		else:
			core_logger.debug(f"{self.__class__.__name__}: listener removed: {listener}")

	def remove_all_listeners(self):
		if self._stopped:
			core_logger.warning(f"{self.__class__.__name__}: removing listeners on a stopped instance")

		listeners_copy = self._listeners_set.copy()
		for listener in listeners_copy:
			self._listener_holder.remove_listener(listener)

		# log
		core_logger.debug(f"{self.__class__.__name__}: removed all listeners")

	#####
	# GrpcMetadata property
	####
	@property
	def grpc_metadata(self) -> list[tuple[str, str]]:
		return [("daemon-client-key", self._client_key)]

	#####
	# other method: manually implemented
	#####
	def attachment_message_list(self, message_id: datatypes.MessageId) -> AsyncIterator[datatypes.Attachment]:
		command_logger.info(f'{self.__class__.__name__}: command: AttachmentMessageList')

		async def iterator(message_iterator: AsyncIterator[commands.AttachmentListResponse]) -> AsyncIterator[
			datatypes.Attachment]:
			async for message in message_iterator:
				for element in message.attachments:
					yield element

		return iterator(self._stubs.attachmentCommandStub.attachment_list(
			commands.AttachmentListRequest(client=self, filter=datatypes.AttachmentFilter(message_id=message_id))))

	#####
	# request stream api (manually implemented)
	#####

	# IdentityCommandService
	async def identity_set_photo(self, file_path: str) -> commands.IdentitySetPhotoResponse:
		if not os.path.isfile(file_path) or not os.access(file_path, os.R_OK):
			raise IOError(f"Cannot open: {file_path}")

		async def identity_set_photo_iterator() -> AsyncIterator:
			with open(file_path, "rb") as fd:
				yield commands.IdentitySetPhotoRequest(
					metadata=commands.IdentitySetPhotoRequestMetadata(filename=os.path.basename(file_path),
																file_size=os.path.getsize(file_path)))
				buffer = fd.read(OlvidClient._CHUNK_LENGTH)
				while len(buffer) > 0:
					yield commands.IdentitySetPhotoRequest(payload=buffer)
					buffer = fd.read(OlvidClient._CHUNK_LENGTH)
		command_logger.info(f'{self.__class__.__name__}: command: IdentitySetPhoto')
		return await self._stubs.identityCommandStub.identity_set_photo(identity_set_photo_iterator())

	# GroupCommandService
	async def group_set_photo(self, group_id: int, file_path: str) -> datatypes.Group:
		async def group_set_photo_iterator() -> AsyncIterator:
			fd = open(file_path, "rb")
			yield commands.GroupSetPhotoRequest(
				metadata=commands.GroupSetPhotoRequestMetadata(group_id=group_id, filename=os.path.basename(file_path),
															file_size=os.path.getsize(file_path)))
			buffer = fd.read(OlvidClient._CHUNK_LENGTH)
			while len(buffer) > 0:
				yield commands.GroupSetPhotoRequest(payload=buffer)
				buffer = fd.read(OlvidClient._CHUNK_LENGTH)
			fd.close()
		command_logger.info(f'{self.__class__.__name__}: command: GroupSetPhoto')
		return (await self._stubs.groupCommandStub.group_set_photo(group_set_photo_iterator())).group

	# MessageCommandService
	async def message_send_with_attachments_files(self, discussion_id: int, file_paths: list[str], body: str = "", reply_id: datatypes.MessageId = None, ephemerality: datatypes.MessageEphemerality = None, disable_link_preview: bool = False) -> tuple[datatypes.Message, list[datatypes.Attachment]]:
		async def message_send_with_attachments_files_generator():
			# send metadata
			files: list[commands.MessageSendWithAttachmentsRequestMetadata.File] = [commands.MessageSendWithAttachmentsRequestMetadata.File(filename=os.path.basename(file_path), file_size=os.path.getsize(file_path)) for file_path in file_paths]
			m = commands.MessageSendWithAttachmentsRequest(metadata=commands.MessageSendWithAttachmentsRequestMetadata(
				body=body if body else "",
				reply_id=reply_id,
				discussion_id=discussion_id,
				ephemerality=ephemerality,
				files=files,
				disable_link_preview=disable_link_preview
			))
			yield m
			# send files content
			for file_path in file_paths:
				with open(file_path, "rb") as fd:
					buffer = fd.read(OlvidClient._CHUNK_LENGTH)
					while len(buffer) > 0:
						yield commands.MessageSendWithAttachmentsRequest(payload=buffer)
						buffer = fd.read(OlvidClient._CHUNK_LENGTH)
				# send file delimiter
				yield commands.MessageSendWithAttachmentsRequest(file_delimiter=True)
		command_logger.info(f'{self.__class__.__name__}: command: MessageSendWithAttachmentsFiles')
		response = await self._stubs.messageCommandStub.message_send_with_attachments(message_send_with_attachments_request_iterator=message_send_with_attachments_files_generator())
		return response.message, response.attachments

	async def message_send_with_attachments(self, discussion_id: int, attachments_filename_with_payload: list[tuple[str, bytes]], body: str = "", reply_id: datatypes.MessageId = None, ephemerality: datatypes.MessageEphemerality = None, disable_link_preview: bool = False) -> tuple[datatypes.Message, list[datatypes.Attachment]]:
		async def message_send_with_attachments_generator():
			# send metadata
			files: list[commands.MessageSendWithAttachmentsRequestMetadata.File] = [commands.MessageSendWithAttachmentsRequestMetadata.File(filename=filename, file_size=len(file_content)) for filename, file_content in attachments_filename_with_payload]
			m = commands.MessageSendWithAttachmentsRequest(metadata=commands.MessageSendWithAttachmentsRequestMetadata(
				body=body if body else "",
				reply_id=reply_id,
				discussion_id=discussion_id,
				ephemerality=ephemerality,
				files=files,
				disable_link_preview=disable_link_preview
			))
			yield m
			# send files content
			for filename, file_content in attachments_filename_with_payload:
				while len(file_content) > 0:
					yield commands.MessageSendWithAttachmentsRequest(payload=file_content[0:self._CHUNK_LENGTH])
					file_content = file_content[self._CHUNK_LENGTH:]
				# send file delimiter
				yield commands.MessageSendWithAttachmentsRequest(file_delimiter=True)
		command_logger.info(f'{self.__class__.__name__}: command: MessageSendWithAttachments')
		response = await self._stubs.messageCommandStub.message_send_with_attachments(message_send_with_attachments_request_iterator=message_send_with_attachments_generator())
		return response.message, response.attachments

	# response stream and non stream api, generated code
	####################################################################################################################
	##### WARNING: DO NOT EDIT: this code is automatically generated, see overlay_generator/generate_olvid_client_code.py
	####################################################################################################################
	# IdentityCommandService
	async def identity_get(self) -> datatypes.Identity:
		command_logger.info(f'{self.__class__.__name__}: command: IdentityGet')
		response: commands.IdentityGetResponse = await self._stubs.identityCommandStub.identity_get(commands.IdentityGetRequest(client=self))
		return response.identity
	
	async def identity_update_details(self, new_details: datatypes.IdentityDetails) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: IdentityUpdateDetails')
		await self._stubs.identityCommandStub.identity_update_details(commands.IdentityUpdateDetailsRequest(client=self, new_details=new_details))
	
	async def identity_remove_photo(self) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: IdentityRemovePhoto')
		await self._stubs.identityCommandStub.identity_remove_photo(commands.IdentityRemovePhotoRequest(client=self))
	
	# identity_set_photo: cannot generate request stream rpc code
	
	async def identity_keycloak_bind(self, configuration_link: str) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: IdentityKeycloakBind')
		await self._stubs.identityCommandStub.identity_keycloak_bind(commands.IdentityKeycloakBindRequest(client=self, configuration_link=configuration_link))
	
	async def identity_keycloak_unbind(self) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: IdentityKeycloakUnbind')
		await self._stubs.identityCommandStub.identity_keycloak_unbind(commands.IdentityKeycloakUnbindRequest(client=self))
	
	async def identity_set_api_key(self, api_key: str) -> datatypes.Identity.ApiKey:
		command_logger.info(f'{self.__class__.__name__}: command: IdentitySetApiKey')
		response: commands.IdentitySetApiKeyResponse = await self._stubs.identityCommandStub.identity_set_api_key(commands.IdentitySetApiKeyRequest(client=self, api_key=api_key))
		return response.api_key
	
	async def identity_set_configuration_link(self, configuration_link: str) -> datatypes.Identity.ApiKey:
		command_logger.info(f'{self.__class__.__name__}: command: IdentitySetConfigurationLink')
		response: commands.IdentitySetConfigurationLinkResponse = await self._stubs.identityCommandStub.identity_set_configuration_link(commands.IdentitySetConfigurationLinkRequest(client=self, configuration_link=configuration_link))
		return response.api_key
	
	# InvitationCommandService
	def invitation_list(self, filter: datatypes.InvitationFilter = None) -> AsyncIterator[datatypes.Invitation]:
		command_logger.info(f'{self.__class__.__name__}: command: InvitationList')
	
		async def iterator(message_iterator: AsyncIterator[commands.InvitationListResponse]) -> AsyncIterator[datatypes.Invitation]:
			async for message in message_iterator:
				for element in message.invitations:
					yield element
		return iterator(self._stubs.invitationCommandStub.invitation_list(commands.InvitationListRequest(client=self, filter=filter)))
	
	async def invitation_get(self, invitation_id: int) -> datatypes.Invitation:
		command_logger.info(f'{self.__class__.__name__}: command: InvitationGet')
		response: commands.InvitationGetResponse = await self._stubs.invitationCommandStub.invitation_get(commands.InvitationGetRequest(client=self, invitation_id=invitation_id))
		return response.invitation
	
	async def invitation_new(self, invitation_url: str) -> datatypes.Invitation:
		command_logger.info(f'{self.__class__.__name__}: command: InvitationNew')
		response: commands.InvitationNewResponse = await self._stubs.invitationCommandStub.invitation_new(commands.InvitationNewRequest(client=self, invitation_url=invitation_url))
		return response.invitation
	
	async def invitation_accept(self, invitation_id: int) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: InvitationAccept')
		await self._stubs.invitationCommandStub.invitation_accept(commands.InvitationAcceptRequest(client=self, invitation_id=invitation_id))
	
	async def invitation_decline(self, invitation_id: int) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: InvitationDecline')
		await self._stubs.invitationCommandStub.invitation_decline(commands.InvitationDeclineRequest(client=self, invitation_id=invitation_id))
	
	async def invitation_sas(self, invitation_id: int, sas: str) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: InvitationSas')
		await self._stubs.invitationCommandStub.invitation_sas(commands.InvitationSasRequest(client=self, invitation_id=invitation_id, sas=sas))
	
	async def invitation_delete(self, invitation_id: int) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: InvitationDelete')
		await self._stubs.invitationCommandStub.invitation_delete(commands.InvitationDeleteRequest(client=self, invitation_id=invitation_id))
	
	# ContactCommandService
	def contact_list(self, filter: datatypes.ContactFilter = None) -> AsyncIterator[datatypes.Contact]:
		command_logger.info(f'{self.__class__.__name__}: command: ContactList')
	
		async def iterator(message_iterator: AsyncIterator[commands.ContactListResponse]) -> AsyncIterator[datatypes.Contact]:
			async for message in message_iterator:
				for element in message.contacts:
					yield element
		return iterator(self._stubs.contactCommandStub.contact_list(commands.ContactListRequest(client=self, filter=filter)))
	
	async def contact_get(self, contact_id: int) -> datatypes.Contact:
		command_logger.info(f'{self.__class__.__name__}: command: ContactGet')
		response: commands.ContactGetResponse = await self._stubs.contactCommandStub.contact_get(commands.ContactGetRequest(client=self, contact_id=contact_id))
		return response.contact
	
	async def contact_delete(self, contact_id: int) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: ContactDelete')
		await self._stubs.contactCommandStub.contact_delete(commands.ContactDeleteRequest(client=self, contact_id=contact_id))
	
	async def contact_introduction(self, first_contact_id: int, second_contact_id: int) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: ContactIntroduction')
		await self._stubs.contactCommandStub.contact_introduction(commands.ContactIntroductionRequest(client=self, first_contact_id=first_contact_id, second_contact_id=second_contact_id))
	
	async def contact_invite_to_one_to_one_discussion(self, contact_id: int) -> datatypes.Invitation:
		command_logger.info(f'{self.__class__.__name__}: command: ContactInviteToOneToOneDiscussion')
		response: commands.ContactInviteToOneToOneDiscussionResponse = await self._stubs.contactCommandStub.contact_invite_to_one_to_one_discussion(commands.ContactInviteToOneToOneDiscussionRequest(client=self, contact_id=contact_id))
		return response.invitation
	
	async def contact_downgrade_one_to_one_discussion(self, contact_id: int) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: ContactDowngradeOneToOneDiscussion')
		await self._stubs.contactCommandStub.contact_downgrade_one_to_one_discussion(commands.ContactDowngradeOneToOneDiscussionRequest(client=self, contact_id=contact_id))
	
	# GroupCommandService
	def group_list(self, filter: datatypes.GroupFilter = None) -> AsyncIterator[datatypes.Group]:
		command_logger.info(f'{self.__class__.__name__}: command: GroupList')
	
		async def iterator(message_iterator: AsyncIterator[commands.GroupListResponse]) -> AsyncIterator[datatypes.Group]:
			async for message in message_iterator:
				for element in message.groups:
					yield element
		return iterator(self._stubs.groupCommandStub.group_list(commands.GroupListRequest(client=self, filter=filter)))
	
	async def group_get(self, group_id: int) -> datatypes.Group:
		command_logger.info(f'{self.__class__.__name__}: command: GroupGet')
		response: commands.GroupGetResponse = await self._stubs.groupCommandStub.group_get(commands.GroupGetRequest(client=self, group_id=group_id))
		return response.group
	
	async def group_new_standard_group(self, name: str = "", description: str = "", admin_contact_ids: list[int] = ()) -> datatypes.Group:
		command_logger.info(f'{self.__class__.__name__}: command: GroupNewStandardGroup')
		response: commands.GroupNewStandardGroupResponse = await self._stubs.groupCommandStub.group_new_standard_group(commands.GroupNewStandardGroupRequest(client=self, name=name, description=description, admin_contact_ids=admin_contact_ids))
		return response.group
	
	async def group_new_controlled_group(self, name: str = "", description: str = "", admin_contact_ids: list[int] = (), contact_ids: list[int] = ()) -> datatypes.Group:
		command_logger.info(f'{self.__class__.__name__}: command: GroupNewControlledGroup')
		response: commands.GroupNewControlledGroupResponse = await self._stubs.groupCommandStub.group_new_controlled_group(commands.GroupNewControlledGroupRequest(client=self, name=name, description=description, admin_contact_ids=admin_contact_ids, contact_ids=contact_ids))
		return response.group
	
	async def group_new_read_only_group(self, name: str = "", description: str = "", admin_contact_ids: list[int] = (), contact_ids: list[int] = ()) -> datatypes.Group:
		command_logger.info(f'{self.__class__.__name__}: command: GroupNewReadOnlyGroup')
		response: commands.GroupNewReadOnlyGroupResponse = await self._stubs.groupCommandStub.group_new_read_only_group(commands.GroupNewReadOnlyGroupRequest(client=self, name=name, description=description, admin_contact_ids=admin_contact_ids, contact_ids=contact_ids))
		return response.group
	
	async def group_new_advanced_group(self, name: str = "", description: str = "", advanced_configuration: datatypes.Group.AdvancedConfiguration = None, members: list[datatypes.GroupMember] = None) -> datatypes.Group:
		command_logger.info(f'{self.__class__.__name__}: command: GroupNewAdvancedGroup')
		response: commands.GroupNewAdvancedGroupResponse = await self._stubs.groupCommandStub.group_new_advanced_group(commands.GroupNewAdvancedGroupRequest(client=self, name=name, description=description, advanced_configuration=advanced_configuration, members=members))
		return response.group
	
	async def group_disband(self, group_id: int) -> datatypes.Group:
		command_logger.info(f'{self.__class__.__name__}: command: GroupDisband')
		response: commands.GroupDisbandResponse = await self._stubs.groupCommandStub.group_disband(commands.GroupDisbandRequest(client=self, group_id=group_id))
		return response.group
	
	async def group_leave(self, group_id: int) -> datatypes.Group:
		command_logger.info(f'{self.__class__.__name__}: command: GroupLeave')
		response: commands.GroupLeaveResponse = await self._stubs.groupCommandStub.group_leave(commands.GroupLeaveRequest(client=self, group_id=group_id))
		return response.group
	
	async def group_update(self, group: datatypes.Group) -> datatypes.Group:
		command_logger.info(f'{self.__class__.__name__}: command: GroupUpdate')
		response: commands.GroupUpdateResponse = await self._stubs.groupCommandStub.group_update(commands.GroupUpdateRequest(client=self, group=group))
		return response.group
	
	async def group_unset_photo(self, group_id: int) -> datatypes.Group:
		command_logger.info(f'{self.__class__.__name__}: command: GroupUnsetPhoto')
		response: commands.GroupUnsetPhotoResponse = await self._stubs.groupCommandStub.group_unset_photo(commands.GroupUnsetPhotoRequest(client=self, group_id=group_id))
		return response.group
	
	# group_set_photo: cannot generate request stream rpc code
	
	# DiscussionCommandService
	def discussion_list(self, filter: datatypes.DiscussionFilter = None) -> AsyncIterator[datatypes.Discussion]:
		command_logger.info(f'{self.__class__.__name__}: command: DiscussionList')
	
		async def iterator(message_iterator: AsyncIterator[commands.DiscussionListResponse]) -> AsyncIterator[datatypes.Discussion]:
			async for message in message_iterator:
				for element in message.discussions:
					yield element
		return iterator(self._stubs.discussionCommandStub.discussion_list(commands.DiscussionListRequest(client=self, filter=filter)))
	
	async def discussion_get(self, discussion_id: int) -> datatypes.Discussion:
		command_logger.info(f'{self.__class__.__name__}: command: DiscussionGet')
		response: commands.DiscussionGetResponse = await self._stubs.discussionCommandStub.discussion_get(commands.DiscussionGetRequest(client=self, discussion_id=discussion_id))
		return response.discussion
	
	async def discussion_get_by_contact(self, contact_id: int) -> datatypes.Discussion:
		command_logger.info(f'{self.__class__.__name__}: command: DiscussionGetByContact')
		response: commands.DiscussionGetByContactResponse = await self._stubs.discussionCommandStub.discussion_get_by_contact(commands.DiscussionGetByContactRequest(client=self, contact_id=contact_id))
		return response.discussion
	
	async def discussion_get_by_group(self, group_id: int) -> datatypes.Discussion:
		command_logger.info(f'{self.__class__.__name__}: command: DiscussionGetByGroup')
		response: commands.DiscussionGetByGroupResponse = await self._stubs.discussionCommandStub.discussion_get_by_group(commands.DiscussionGetByGroupRequest(client=self, group_id=group_id))
		return response.discussion
	
	async def discussion_empty(self, discussion_id: int, delete_everywhere: bool = False) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: DiscussionEmpty')
		await self._stubs.discussionCommandStub.discussion_empty(commands.DiscussionEmptyRequest(client=self, discussion_id=discussion_id, delete_everywhere=delete_everywhere))
	
	async def discussion_settings_get(self, discussion_id: int) -> datatypes.DiscussionSettings:
		command_logger.info(f'{self.__class__.__name__}: command: DiscussionSettingsGet')
		response: commands.DiscussionSettingsGetResponse = await self._stubs.discussionCommandStub.discussion_settings_get(commands.DiscussionSettingsGetRequest(client=self, discussion_id=discussion_id))
		return response.settings
	
	async def discussion_settings_set(self, settings: datatypes.DiscussionSettings) -> datatypes.DiscussionSettings:
		command_logger.info(f'{self.__class__.__name__}: command: DiscussionSettingsSet')
		response: commands.DiscussionSettingsSetResponse = await self._stubs.discussionCommandStub.discussion_settings_set(commands.DiscussionSettingsSetRequest(client=self, settings=settings))
		return response.new_settings
	
	def discussion_locked_list(self) -> AsyncIterator[datatypes.Discussion]:
		command_logger.info(f'{self.__class__.__name__}: command: DiscussionLockedList')
	
		async def iterator(message_iterator: AsyncIterator[commands.DiscussionLockedListResponse]) -> AsyncIterator[datatypes.Discussion]:
			async for message in message_iterator:
				for element in message.discussions:
					yield element
		return iterator(self._stubs.discussionCommandStub.discussion_locked_list(commands.DiscussionLockedListRequest(client=self)))
	
	async def discussion_locked_delete(self, discussion_id: int) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: DiscussionLockedDelete')
		await self._stubs.discussionCommandStub.discussion_locked_delete(commands.DiscussionLockedDeleteRequest(client=self, discussion_id=discussion_id))
	
	# MessageCommandService
	def message_list(self, filter: datatypes.MessageFilter = None, unread: bool = False) -> AsyncIterator[datatypes.Message]:
		command_logger.info(f'{self.__class__.__name__}: command: MessageList')
	
		async def iterator(message_iterator: AsyncIterator[commands.MessageListResponse]) -> AsyncIterator[datatypes.Message]:
			async for message in message_iterator:
				for element in message.messages:
					yield element
		return iterator(self._stubs.messageCommandStub.message_list(commands.MessageListRequest(client=self, filter=filter, unread=unread)))
	
	async def message_get(self, message_id: datatypes.MessageId) -> datatypes.Message:
		command_logger.info(f'{self.__class__.__name__}: command: MessageGet')
		response: commands.MessageGetResponse = await self._stubs.messageCommandStub.message_get(commands.MessageGetRequest(client=self, message_id=message_id))
		return response.message
	
	async def message_refresh(self) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: MessageRefresh')
		await self._stubs.messageCommandStub.message_refresh(commands.MessageRefreshRequest(client=self))
	
	async def message_delete(self, message_id: datatypes.MessageId, delete_everywhere: bool = False) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: MessageDelete')
		await self._stubs.messageCommandStub.message_delete(commands.MessageDeleteRequest(client=self, message_id=message_id, delete_everywhere=delete_everywhere))
	
	async def message_send(self, discussion_id: int, body: str, reply_id: datatypes.MessageId = None, ephemerality: datatypes.MessageEphemerality = None, disable_link_preview: bool = False) -> datatypes.Message:
		command_logger.info(f'{self.__class__.__name__}: command: MessageSend')
		response: commands.MessageSendResponse = await self._stubs.messageCommandStub.message_send(commands.MessageSendRequest(client=self, discussion_id=discussion_id, body=body, reply_id=reply_id, ephemerality=ephemerality, disable_link_preview=disable_link_preview))
		return response.message
	
	# message_send_with_attachments: cannot generate request stream rpc code
	
	async def message_react(self, message_id: datatypes.MessageId, reaction: str) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: MessageReact')
		await self._stubs.messageCommandStub.message_react(commands.MessageReactRequest(client=self, message_id=message_id, reaction=reaction))
	
	async def message_update_body(self, message_id: datatypes.MessageId, updated_body: str) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: MessageUpdateBody')
		await self._stubs.messageCommandStub.message_update_body(commands.MessageUpdateBodyRequest(client=self, message_id=message_id, updated_body=updated_body))
	
	async def message_send_voip(self, discussion_id: int) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: MessageSendVoip')
		await self._stubs.messageCommandStub.message_send_voip(commands.MessageSendVoipRequest(client=self, discussion_id=discussion_id))
	
	# AttachmentCommandService
	def attachment_list(self, filter: datatypes.AttachmentFilter = None) -> AsyncIterator[datatypes.Attachment]:
		command_logger.info(f'{self.__class__.__name__}: command: AttachmentList')
	
		async def iterator(message_iterator: AsyncIterator[commands.AttachmentListResponse]) -> AsyncIterator[datatypes.Attachment]:
			async for message in message_iterator:
				for element in message.attachments:
					yield element
		return iterator(self._stubs.attachmentCommandStub.attachment_list(commands.AttachmentListRequest(client=self, filter=filter)))
	
	async def attachment_get(self, attachment_id: datatypes.AttachmentId) -> datatypes.Attachment:
		command_logger.info(f'{self.__class__.__name__}: command: AttachmentGet')
		response: commands.AttachmentGetResponse = await self._stubs.attachmentCommandStub.attachment_get(commands.AttachmentGetRequest(client=self, attachment_id=attachment_id))
		return response.attachment
	
	async def attachment_delete(self, attachment_id: datatypes.AttachmentId, delete_everywhere: bool = False) -> None:
		command_logger.info(f'{self.__class__.__name__}: command: AttachmentDelete')
		await self._stubs.attachmentCommandStub.attachment_delete(commands.AttachmentDeleteRequest(client=self, attachment_id=attachment_id, delete_everywhere=delete_everywhere))
	
	def attachment_download(self, attachment_id: datatypes.AttachmentId) -> AsyncIterator[bytes]:
		command_logger.info(f'{self.__class__.__name__}: command: AttachmentDownload')
	
		async def iterator(message_iterator: AsyncIterator[commands.AttachmentDownloadResponse]) -> AsyncIterator[bytes]:
			async for message in message_iterator:
				yield message.chunk
		return iterator(self._stubs.attachmentCommandStub.attachment_download(commands.AttachmentDownloadRequest(client=self, attachment_id=attachment_id)))
	
	# StorageCommandService
	def storage_list(self, filter: datatypes.StorageElementFilter = None) -> AsyncIterator[datatypes.StorageElement]:
		command_logger.info(f'{self.__class__.__name__}: command: StorageList')
	
		async def iterator(message_iterator: AsyncIterator[commands.StorageListResponse]) -> AsyncIterator[datatypes.StorageElement]:
			async for message in message_iterator:
				for element in message.elements:
					yield element
		return iterator(self._stubs.storageCommandStub.storage_list(commands.StorageListRequest(client=self, filter=filter)))
	
	async def storage_get(self, key: str) -> str:
		command_logger.info(f'{self.__class__.__name__}: command: StorageGet')
		response: commands.StorageGetResponse = await self._stubs.storageCommandStub.storage_get(commands.StorageGetRequest(client=self, key=key))
		return response.value
	
	async def storage_set(self, key: str, value: str) -> str:
		command_logger.info(f'{self.__class__.__name__}: command: StorageSet')
		response: commands.StorageSetResponse = await self._stubs.storageCommandStub.storage_set(commands.StorageSetRequest(client=self, key=key, value=value))
		return response.previous_value
	
	async def storage_unset(self, key: str) -> str:
		command_logger.info(f'{self.__class__.__name__}: command: StorageUnset')
		response: commands.StorageUnsetResponse = await self._stubs.storageCommandStub.storage_unset(commands.StorageUnsetRequest(client=self, key=key))
		return response.previous_value
	
	# DiscussionStorageCommandService
	def discussion_storage_list(self, discussion_id: int, filter: datatypes.StorageElementFilter = None) -> AsyncIterator[datatypes.StorageElement]:
		command_logger.info(f'{self.__class__.__name__}: command: DiscussionStorageList')
	
		async def iterator(message_iterator: AsyncIterator[commands.DiscussionStorageListResponse]) -> AsyncIterator[datatypes.StorageElement]:
			async for message in message_iterator:
				for element in message.elements:
					yield element
		return iterator(self._stubs.discussionStorageCommandStub.discussion_storage_list(commands.DiscussionStorageListRequest(client=self, discussion_id=discussion_id, filter=filter)))
	
	async def discussion_storage_get(self, discussion_id: int, key: str) -> str:
		command_logger.info(f'{self.__class__.__name__}: command: DiscussionStorageGet')
		response: commands.DiscussionStorageGetResponse = await self._stubs.discussionStorageCommandStub.discussion_storage_get(commands.DiscussionStorageGetRequest(client=self, discussion_id=discussion_id, key=key))
		return response.value
	
	async def discussion_storage_set(self, discussion_id: int, key: str, value: str) -> str:
		command_logger.info(f'{self.__class__.__name__}: command: DiscussionStorageSet')
		response: commands.DiscussionStorageSetResponse = await self._stubs.discussionStorageCommandStub.discussion_storage_set(commands.DiscussionStorageSetRequest(client=self, discussion_id=discussion_id, key=key, value=value))
		return response.previous_value
	
	async def discussion_storage_unset(self, discussion_id: int, key: str) -> str:
		command_logger.info(f'{self.__class__.__name__}: command: DiscussionStorageUnset')
		response: commands.DiscussionStorageUnsetResponse = await self._stubs.discussionStorageCommandStub.discussion_storage_unset(commands.DiscussionStorageUnsetRequest(client=self, discussion_id=discussion_id, key=key))
		return response.previous_value
	
	# InvitationNotificationService
	def _notif_invitation_received(self) -> AsyncIterator[notifications.InvitationReceivedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: InvitationReceived')
		return self._stubs.invitationNotificationStub.invitation_received(notifications.SubscribeToInvitationReceivedNotification(client=self))
	
	def _notif_invitation_sent(self) -> AsyncIterator[notifications.InvitationSentNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: InvitationSent')
		return self._stubs.invitationNotificationStub.invitation_sent(notifications.SubscribeToInvitationSentNotification(client=self))
	
	def _notif_invitation_deleted(self) -> AsyncIterator[notifications.InvitationDeletedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: InvitationDeleted')
		return self._stubs.invitationNotificationStub.invitation_deleted(notifications.SubscribeToInvitationDeletedNotification(client=self))
	
	def _notif_invitation_updated(self) -> AsyncIterator[notifications.InvitationUpdatedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: InvitationUpdated')
		return self._stubs.invitationNotificationStub.invitation_updated(notifications.SubscribeToInvitationUpdatedNotification(client=self))
	
	# ContactNotificationService
	def _notif_contact_new(self) -> AsyncIterator[notifications.ContactNewNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: ContactNew')
		return self._stubs.contactNotificationStub.contact_new(notifications.SubscribeToContactNewNotification(client=self))
	
	def _notif_contact_deleted(self) -> AsyncIterator[notifications.ContactDeletedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: ContactDeleted')
		return self._stubs.contactNotificationStub.contact_deleted(notifications.SubscribeToContactDeletedNotification(client=self))
	
	def _notif_contact_details_updated(self) -> AsyncIterator[notifications.ContactDetailsUpdatedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: ContactDetailsUpdated')
		return self._stubs.contactNotificationStub.contact_details_updated(notifications.SubscribeToContactDetailsUpdatedNotification(client=self))
	
	# GroupNotificationService
	def _notif_group_new(self) -> AsyncIterator[notifications.GroupNewNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: GroupNew')
		return self._stubs.groupNotificationStub.group_new(notifications.SubscribeToGroupNewNotification(client=self))
	
	def _notif_group_deleted(self) -> AsyncIterator[notifications.GroupDeletedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: GroupDeleted')
		return self._stubs.groupNotificationStub.group_deleted(notifications.SubscribeToGroupDeletedNotification(client=self))
	
	def _notif_group_name_updated(self) -> AsyncIterator[notifications.GroupNameUpdatedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: GroupNameUpdated')
		return self._stubs.groupNotificationStub.group_name_updated(notifications.SubscribeToGroupNameUpdatedNotification(client=self))
	
	def _notif_group_description_updated(self) -> AsyncIterator[notifications.GroupDescriptionUpdatedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: GroupDescriptionUpdated')
		return self._stubs.groupNotificationStub.group_description_updated(notifications.SubscribeToGroupDescriptionUpdatedNotification(client=self))
	
	def _notif_group_pending_member_added(self) -> AsyncIterator[notifications.GroupPendingMemberAddedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: GroupPendingMemberAdded')
		return self._stubs.groupNotificationStub.group_pending_member_added(notifications.SubscribeToGroupPendingMemberAddedNotification(client=self))
	
	def _notif_group_pending_member_removed(self) -> AsyncIterator[notifications.GroupPendingMemberRemovedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: GroupPendingMemberRemoved')
		return self._stubs.groupNotificationStub.group_pending_member_removed(notifications.SubscribeToGroupPendingMemberRemovedNotification(client=self))
	
	def _notif_group_member_joined(self) -> AsyncIterator[notifications.GroupMemberJoinedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: GroupMemberJoined')
		return self._stubs.groupNotificationStub.group_member_joined(notifications.SubscribeToGroupMemberJoinedNotification(client=self))
	
	def _notif_group_member_left(self) -> AsyncIterator[notifications.GroupMemberLeftNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: GroupMemberLeft')
		return self._stubs.groupNotificationStub.group_member_left(notifications.SubscribeToGroupMemberLeftNotification(client=self))
	
	def _notif_group_own_permissions_updated(self) -> AsyncIterator[notifications.GroupOwnPermissionsUpdatedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: GroupOwnPermissionsUpdated')
		return self._stubs.groupNotificationStub.group_own_permissions_updated(notifications.SubscribeToGroupOwnPermissionsUpdatedNotification(client=self))
	
	def _notif_group_member_permissions_updated(self) -> AsyncIterator[notifications.GroupMemberPermissionsUpdatedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: GroupMemberPermissionsUpdated')
		return self._stubs.groupNotificationStub.group_member_permissions_updated(notifications.SubscribeToGroupMemberPermissionsUpdatedNotification(client=self))
	
	def _notif_group_update_in_progress(self) -> AsyncIterator[notifications.GroupUpdateInProgressNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: GroupUpdateInProgress')
		return self._stubs.groupNotificationStub.group_update_in_progress(notifications.SubscribeToGroupUpdateInProgressNotification(client=self))
	
	def _notif_group_update_finished(self) -> AsyncIterator[notifications.GroupUpdateFinishedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: GroupUpdateFinished')
		return self._stubs.groupNotificationStub.group_update_finished(notifications.SubscribeToGroupUpdateFinishedNotification(client=self))
	
	# DiscussionNotificationService
	def _notif_discussion_new(self) -> AsyncIterator[notifications.DiscussionNewNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: DiscussionNew')
		return self._stubs.discussionNotificationStub.discussion_new(notifications.SubscribeToDiscussionNewNotification(client=self))
	
	def _notif_discussion_locked(self) -> AsyncIterator[notifications.DiscussionLockedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: DiscussionLocked')
		return self._stubs.discussionNotificationStub.discussion_locked(notifications.SubscribeToDiscussionLockedNotification(client=self))
	
	def _notif_discussion_title_updated(self) -> AsyncIterator[notifications.DiscussionTitleUpdatedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: DiscussionTitleUpdated')
		return self._stubs.discussionNotificationStub.discussion_title_updated(notifications.SubscribeToDiscussionTitleUpdatedNotification(client=self))
	
	def _notif_discussion_settings_updated(self) -> AsyncIterator[notifications.DiscussionSettingsUpdatedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: DiscussionSettingsUpdated')
		return self._stubs.discussionNotificationStub.discussion_settings_updated(notifications.SubscribeToDiscussionSettingsUpdatedNotification(client=self))
	
	# MessageNotificationService
	def _notif_message_received(self) -> AsyncIterator[notifications.MessageReceivedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: MessageReceived')
		return self._stubs.messageNotificationStub.message_received(notifications.SubscribeToMessageReceivedNotification(client=self))
	
	def _notif_message_sent(self) -> AsyncIterator[notifications.MessageSentNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: MessageSent')
		return self._stubs.messageNotificationStub.message_sent(notifications.SubscribeToMessageSentNotification(client=self))
	
	def _notif_message_deleted(self) -> AsyncIterator[notifications.MessageDeletedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: MessageDeleted')
		return self._stubs.messageNotificationStub.message_deleted(notifications.SubscribeToMessageDeletedNotification(client=self))
	
	def _notif_message_body_updated(self) -> AsyncIterator[notifications.MessageBodyUpdatedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: MessageBodyUpdated')
		return self._stubs.messageNotificationStub.message_body_updated(notifications.SubscribeToMessageBodyUpdatedNotification(client=self))
	
	def _notif_message_uploaded(self) -> AsyncIterator[notifications.MessageUploadedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: MessageUploaded')
		return self._stubs.messageNotificationStub.message_uploaded(notifications.SubscribeToMessageUploadedNotification(client=self))
	
	def _notif_message_delivered(self) -> AsyncIterator[notifications.MessageDeliveredNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: MessageDelivered')
		return self._stubs.messageNotificationStub.message_delivered(notifications.SubscribeToMessageDeliveredNotification(client=self))
	
	def _notif_message_read(self) -> AsyncIterator[notifications.MessageReadNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: MessageRead')
		return self._stubs.messageNotificationStub.message_read(notifications.SubscribeToMessageReadNotification(client=self))
	
	def _notif_message_location_received(self) -> AsyncIterator[notifications.MessageLocationReceivedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: MessageLocationReceived')
		return self._stubs.messageNotificationStub.message_location_received(notifications.SubscribeToMessageLocationReceivedNotification(client=self))
	
	def _notif_message_location_sharing_start(self) -> AsyncIterator[notifications.MessageLocationSharingStartNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: MessageLocationSharingStart')
		return self._stubs.messageNotificationStub.message_location_sharing_start(notifications.SubscribeToMessageLocationSharingStartNotification(client=self))
	
	def _notif_message_location_sharing_update(self) -> AsyncIterator[notifications.MessageLocationSharingUpdateNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: MessageLocationSharingUpdate')
		return self._stubs.messageNotificationStub.message_location_sharing_update(notifications.SubscribeToMessageLocationSharingUpdateNotification(client=self))
	
	def _notif_message_location_sharing_end(self) -> AsyncIterator[notifications.MessageLocationSharingEndNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: MessageLocationSharingEnd')
		return self._stubs.messageNotificationStub.message_location_sharing_end(notifications.SubscribeToMessageLocationSharingEndNotification(client=self))
	
	def _notif_message_reaction_added(self) -> AsyncIterator[notifications.MessageReactionAddedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: MessageReactionAdded')
		return self._stubs.messageNotificationStub.message_reaction_added(notifications.SubscribeToMessageReactionAddedNotification(client=self))
	
	def _notif_message_reaction_updated(self) -> AsyncIterator[notifications.MessageReactionUpdatedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: MessageReactionUpdated')
		return self._stubs.messageNotificationStub.message_reaction_updated(notifications.SubscribeToMessageReactionUpdatedNotification(client=self))
	
	def _notif_message_reaction_removed(self) -> AsyncIterator[notifications.MessageReactionRemovedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: MessageReactionRemoved')
		return self._stubs.messageNotificationStub.message_reaction_removed(notifications.SubscribeToMessageReactionRemovedNotification(client=self))
	
	# AttachmentNotificationService
	def _notif_attachment_received(self) -> AsyncIterator[notifications.AttachmentReceivedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: AttachmentReceived')
		return self._stubs.attachmentNotificationStub.attachment_received(notifications.SubscribeToAttachmentReceivedNotification(client=self))
	
	def _notif_attachment_uploaded(self) -> AsyncIterator[notifications.AttachmentUploadedNotification]:
		notification_logger.debug(f'{self.__class__.__name__}: subscribed to: AttachmentUploaded')
		return self._stubs.attachmentNotificationStub.attachment_uploaded(notifications.SubscribeToAttachmentUploadedNotification(client=self))
	
	


import asyncclick as click
from google.protobuf.json_format import Parse, ParseError

from olvid import datatypes
from ..interactive_tree import interactive_tree
from ..tools.ClientSingleton import ClientSingleton
from ..tools.cli_tools import filter_fields_and_print_normal_message, print_error_message
from ..tools.click_wrappers import WrapperGroup

# default permissions
DEFAULT_ADMIN_PERMISSIONS = datatypes.GroupMemberPermissions(admin=True, remote_delete_anything=True,
																edit_or_remote_delete_own_messages=True,
																change_settings=True,
																send_message=True)
DEFAULT_MEMBER_PERMISSIONS = datatypes.GroupMemberPermissions(admin=False, remote_delete_anything=True,
																edit_or_remote_delete_own_messages=True,
																change_settings=False, send_message=True)


#####
# group
#####
@interactive_tree.group("group", help="manage current identity groups", cls=WrapperGroup)
def group_tree():
	pass


#####
# group get
#####
# noinspection PyProtectedMember
@group_tree.command("get", help="list current identity groups")
@click.option("-a", "--all", "get_all", is_flag=True)
@click.argument("group_ids", nargs=-1, type=click.INT)
@click.option("-f", "--fields", "fields", type=str)
@click.option("--filter", "filter_", type=str)
async def group_get(get_all: bool, group_ids: list[int], fields: str, filter_: str = ""):
	# build filter
	group_filter: datatypes.GroupFilter = datatypes.GroupFilter()
	if filter_:
		try:
			parsed_message = Parse(filter_, datatypes.GroupFilter()._to_native(group_filter))
			group_filter = datatypes.GroupFilter._from_native(parsed_message)
		except ParseError as e:
			print_error_message(f"Cannot parse filter: {e}")
			return

	groups: list[datatypes.Group]
	if not group_ids or get_all:
		groups = [group async for group in ClientSingleton.get_client().group_list(filter=group_filter)]
	else:
		groups = [await ClientSingleton.get_client().group_get(group_id=group_id) for group_id in group_ids]

	group_as_strings: list[str] = []
	for group in groups:
		if fields:
			group_as_strings.append(filter_fields_and_print_normal_message(group, fields))
		else:
			group_as_strings.append(await group_to_string(group))

	print(("-" * 20 + "\n").join(group_as_strings))


#####
# group new
#####
@group_tree.group("new", help="create new groups", cls=WrapperGroup)
def group_new_tree():
	pass


#####
# group new standard
#####
@group_new_tree.command("standard", help="create a standard group, where everyone have admin rights")
@click.option("-n", "--name", "group_name", default="")
@click.option("-d", "--description", "group_description", default="")
@click.argument("admin_contact_ids", nargs=-1, type=click.INT, required=False)
async def group_new_standard(admin_contact_ids: tuple[int], group_name: str, group_description: str = "", ):
	group = await ClientSingleton.get_client().group_new_standard_group(
		name=group_name,
		description=group_description,
		admin_contact_ids=list(admin_contact_ids)
	)
	print(await group_to_string(group))


#####
# group new controlled
#####
@group_new_tree.command("controlled", help="create a controlled group with admin and regular members")
@click.option("-n", "--name", "group_name")
@click.option("-d", "--description", "group_description", default="")
@click.argument("contacts_id", nargs=-1, type=click.INT, required=False)
@click.option("-a", "--admin", "admin_contact_ids", multiple=True, type=click.INT)
async def group_new_controlled(contacts_id: tuple[int], admin_contact_ids: tuple[int], group_name: str, group_description: str = "", ):
	group = await ClientSingleton.get_client().group_new_controlled_group(
		name=group_name,
		description=group_description,
		contact_ids=list(contacts_id),
		admin_contact_ids=list(admin_contact_ids)
	)
	print(await group_to_string(group))


#####
# group new read-only
#####
@group_new_tree.command("read-only", help="create a read-only group where only admin members can post messages")
@click.option("-n", "--name", "group_name")
@click.option("-d", "--description", "group_description", default="")
@click.option("-a", "--admin", "admin_contact_ids", multiple=True, type=click.INT)
@click.argument("contacts_id", nargs=-1, type=click.INT, required=False)
async def group_new_read_only(contacts_id: tuple[int], admin_contact_ids: tuple[int], group_name: str, group_description: str = "", ):
	group = await ClientSingleton.get_client().group_new_read_only_group(
		name=group_name,
		description=group_description,
		contact_ids=list(contacts_id),
		admin_contact_ids=list(admin_contact_ids)
	)
	print(await group_to_string(group))


#####
# group new advanced
#####
@group_new_tree.command("advanced", help="create an advanced group with custom permissions for members (not recommended)")
@click.option("-n", "--name", "group_name")
@click.option("-d", "--description", "group_description", default="")
@click.argument("contacts_id", nargs=-1, type=click.INT, required=False)
@click.option("--read-only", "read_only", is_flag=True, help="By default new members won't have the send_message permission")
@click.option("--do-not-remote-delete", "do_not_remote_delete", is_flag=True, help="By default do not give remote delete permission to future members")
@click.option("--remote-delete-admins", "remote_delete_admins", is_flag=True, help="By default give future admins remote delete permission")
@click.option("--remote-delete-everyone", "remote_delete_everyone", is_flag=True, help="By default give future member remote delete permission")
@click.option("-a", "--admin", "admin", is_flag=True, help="Set members as admin")
@click.option("-s", "--send-message", "send_message", is_flag=True, help="Let members send message")
@click.option("-e", "--edit", "edit", is_flag=True, help="Let members edit or delete their messages")
@click.option("-r", "--remote-delete", "remote_delete", is_flag=True, help="Let members remote delete every messages")
@click.option("-c", "--change-settings", "change_settings", is_flag=True, help="Let members change discussion settings")
async def group_new_advanced(contacts_id: tuple[int], read_only: bool, do_not_remote_delete: bool, remote_delete_admins: bool, remote_delete_everyone: bool, group_name: str, group_description: str, admin: bool, send_message: bool, edit: bool, remote_delete: bool, change_settings: bool):
	if (do_not_remote_delete and (remote_delete_admins or remote_delete_everyone)) or (remote_delete_admins and remote_delete_everyone):
		print_error_message("Can only pass one global remote delete parameter option")
		return

	rd = datatypes.Group.AdvancedConfiguration.RemoteDelete
	global_remote_delete: rd = rd.REMOTE_DELETE_UNSPECIFIED
	if do_not_remote_delete:
		global_remote_delete = rd.REMOTE_DELETE_NOBODY
	elif remote_delete_admins:
		global_remote_delete = rd.REMOTE_DELETE_ADMINS
	elif remote_delete_everyone:
		global_remote_delete = rd.REMOTE_DELETE_EVERYONE

	advanced_configuration: datatypes.Group.AdvancedConfiguration = datatypes.Group.AdvancedConfiguration(
		read_only=read_only,
		remote_delete=global_remote_delete
	)

	members: list[datatypes.GroupMember] = []
	for cid in contacts_id:
		members.append(datatypes.GroupMember(contact_id=cid, permissions=datatypes.GroupMemberPermissions(
			admin=admin,
			send_message=send_message,
			remote_delete_anything=edit,
			edit_or_remote_delete_own_messages=remote_delete,
			change_settings=change_settings
		)))

	group = await ClientSingleton.get_client().group_new_advanced_group(
		name=group_name,
		description=group_description,
		advanced_configuration=advanced_configuration,
		members=members,
	)
	print(await group_to_string(group))


#####
# group disband
#####
@group_tree.command("disband", help="disband a group where you have admin permissions")
@click.argument("group_id", type=click.INT)
async def group_disband(group_id):
	group = await ClientSingleton.get_client().group_disband(group_id)
	print(await group_to_string(group))


#####
# group leave
#####
@group_tree.command("leave", help="leave a group")
@click.argument("group_id", type=click.INT)
async def group_leave(group_id):
	group = await ClientSingleton.get_client().group_leave(group_id)
	print(await group_to_string(group))


#####
# group update
#####
@group_tree.command("update", help="update group details and/or members (advanced command)")
@click.option("-n", "--name", "group_name", type=str, default="")
@click.option("-d", "--description", "group_description", type=str, default="")
@click.option("-am", "--add-member", "add_members", multiple=True, type=click.INT)
@click.option("-aam", "--add-admin-member", "add_admin_members", multiple=True, type=click.INT)
@click.option("-dm", "--delete-member", "delete_members", multiple=True, type=click.INT)
@click.argument("group_id", type=click.INT, required=True)
async def group_update(group_id: int, group_name: str, group_description: str, add_admin_members: tuple[int],
						add_members: tuple[int], delete_members: tuple[int]):
	group = await ClientSingleton.get_client().group_get(group_id=group_id)
	if group_name:
		group.name = group_name
	if group_description:
		group.description = group_description

	# add admin members
	for member_id in add_admin_members:
		group.members.append(datatypes.GroupMember(contact_id=member_id, permissions=DEFAULT_ADMIN_PERMISSIONS))

	# add members
	for member_id in add_members:
		group.members.append(datatypes.GroupMember(contact_id=member_id, permissions=DEFAULT_MEMBER_PERMISSIONS))

	# remove members
	group.members = list(filter(lambda m: m.contact_id not in delete_members, group.members))

	updated_group = await ClientSingleton.get_client().group_update(group=group)
	print(await group_to_string(updated_group))


#####
# group admin
#####
@group_tree.command("admin", help="manage group admins")
@click.option("-pa", "--promote-admin", "admins_to_add", multiple=True, type=click.INT)
@click.option("-da", "--demote-admin", "admins_to_remove", multiple=True, type=click.INT)
@click.option("-ppa", "--pending-promote-admin", "pending_admins_to_add", multiple=True, type=click.INT)
@click.option("-pda", "--pending-demote-admin", "pending_admins_to_remove", multiple=True, type=click.INT)
@click.argument("group_id", type=click.INT, required=True)
async def group_update(group_id: int, admins_to_add: tuple[int], admins_to_remove: tuple[int],
						pending_admins_to_add: tuple[int], pending_admins_to_remove: tuple[int]):
	group = await ClientSingleton.get_client().group_get(group_id=group_id)

	for member in group.members:
		if member.contact_id in admins_to_add:
			member.permissions = DEFAULT_ADMIN_PERMISSIONS
		if member.contact_id in admins_to_remove:
			member.permissions = DEFAULT_MEMBER_PERMISSIONS

	for pending in group.pending_members:
		if pending.pending_member_id in pending_admins_to_add:
			pending.permissions = DEFAULT_ADMIN_PERMISSIONS
		if pending.pending_member_id in pending_admins_to_remove:
			pending.permissions = DEFAULT_MEMBER_PERMISSIONS

	updated_group = await ClientSingleton.get_client().group_update(group=group)
	print(await group_to_string(updated_group))


#####
# group permissions
#####
@group_tree.command("permissions", help="manage a group member permissions")
@click.option("-a", "--admin", type=bool)
@click.option("-rda", "--remote-delete-anything", "remote_delete_anything", type=bool, default=False)
@click.option("-erm", "--edit-or-remote-delete-own-messages", "edit_or_remote_delete_own_messages", type=bool,
				default=False)
@click.option("-cs", "--change-settings", "change_settings", type=bool, default=False)
@click.option("-sm", "--send-message", "send_message", type=bool, default=False)
@click.argument("group_id", type=click.INT, required=True)
@click.argument("contact_id", type=click.INT, required=True)
async def group_update(group_id: int, contact_id: int, admin: bool, remote_delete_anything: bool,
						edit_or_remote_delete_own_messages: bool, change_settings: bool, send_message: bool):
	group = await ClientSingleton.get_client().group_get(group_id=group_id)

	found = False
	for member in group.members:
		if member.contact_id == contact_id:
			member.permissions = datatypes.GroupMemberPermissions(
				admin=admin, remote_delete_anything=remote_delete_anything,
				edit_or_remote_delete_own_messages=edit_or_remote_delete_own_messages,
				change_settings=change_settings, send_message=send_message)
			found = True

	if not found:
		raise click.exceptions.BadArgumentUsage("Member not found")

	updated_group = await ClientSingleton.get_client().group_update(group=group)
	print(await group_to_string(updated_group))


#####
# group photo
#####
@group_tree.group("photo", help="get/update group photos", cls=WrapperGroup)
def group_photo_tree():
	pass


#####
# group photo set
#####
@group_photo_tree.command("set")
@click.argument("group_id", required=True, type=click.INT)
@click.argument("photo_path", required=True, type=click.STRING)
async def group_photo_set(group_id: int, photo_path: str):
	try:
		open(photo_path, "rb")
	except IOError:
		raise click.exceptions.FileError("File not found: " + photo_path)

	updated_group = await ClientSingleton.get_client().group_set_photo(group_id, photo_path)
	print(await group_to_string(updated_group))


#####
# group photo unset
#####
@group_photo_tree.command("unset")
@click.argument("group_id", required=True, type=click.INT)
async def group_photo_unset(group_id: int):
	updated_group = await ClientSingleton.get_client().group_unset_photo(group_id=group_id)
	print(await group_to_string(updated_group))


#####
# tools
#####
async def group_to_string(group: datatypes.Group) -> str:
	def permission_to_string(permission: datatypes.GroupMemberPermissions) -> str:
		permissions: list[str] = []
		if permission.admin:
			permissions.append("admin")
		if permission.send_message:
			permissions.append("send-message")
		if permission.edit_or_remote_delete_own_messages:
			permissions.append("edit-or-remote-delete-own")
		if permission.remote_delete_anything:
			permissions.append("remote-delete-anything")
		if permission.change_settings:
			permissions.append("change-settings")
		return "(" + ", ".join(permissions) + ")"
	s = f"{group.id}:{' ' + group.name if group.name else ''} (type: {group.type.name}){' (has a photo)' if group.has_a_photo else ''}\n"
	s += str(group.advanced_configuration) + "\n" if group.type == datatypes.Group.Type.TYPE_ADVANCED else ''
	s += f"{group.description}\n" if group.description else ""
	s += f"(has photo)\n" if group.has_a_photo else ""
	s += f"permissions: {permission_to_string(group.own_permissions)}\n"
	if group.members:
		s += "Members:\n"
	for member in group.members:
		# noinspection PyProtectedMember
		s += f"\t{member.contact_id}: {(await group._client.contact_get(contact_id=member.contact_id)).display_name}: {permission_to_string(member.permissions)}\n"
	if group.pending_members:
		s += "Pending Members:\n"
	for pending_member in group.pending_members:
		s += f"\t{pending_member.contact_id}: {pending_member.display_name}: {permission_to_string(pending_member.permissions)}\n"
	return s



import asyncio
import sys
import traceback

import asyncclick as click
import grpc.aio

from .cli.tools.ClientSingleton import ClientSingleton
from .cli.interactive_tree import interactive_tree
from .cli.root_tree import root_tree

from .cli.tools.cli_tools import print_error_message

# import logging
# logging.basicConfig(level=logging.INFO, format="[%(levelname)5s]: %(message)s", filename="cli.log")
# logging.info("Level set to file")


async def async_main():
	# initialize connection to server
	try:
		# initialize connection to server
		await ClientSingleton.init()
	except ValueError as e:
		click.echo(click.style(e, fg="red"))
		exit(1)
	except grpc.aio.AioRpcError as e:
		click.echo(click.style(e.details(), fg="red"))
		exit(1)

	try:
		try:
			for arg in sys.argv[1:]:
				if not arg.startswith("-"):
					break
				if arg == "--help":
					raise click.exceptions.ClickException("")

			async with await root_tree.make_context("olvid-cli", list(sys.argv[1:])) as ctx:
				await root_tree.invoke(ctx)
		except (click.UsageError, click.ClickException) as e:
			if e.format_message():
				click.echo(click.style(e.format_message(), fg="red"))
			interactive_tree.params.extend(root_tree.params)
			interactive_tree.commands.update(root_tree.commands)
			print(interactive_tree.get_help(interactive_tree.context_class(interactive_tree)))
		except click.exceptions.Exit:
			pass
	except Exception:
		print_error_message("Unexpected exception caught in main_tree")
		traceback.print_exc()


def main():
	# This is supposed to improve performances, and it fix and exception when running cli in docker (not sure why)
	import uvloop
	asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
	asyncio.run(async_main())


if __name__ == "__main__":
	main()


####
# WARNING: DO NOT EDIT: this code is automatically generated, see overlay_generator/generate_protobuf_overlay
####

# noinspection PyCompatibility
from . import commands
from . import admin
from . import notifications
from . import types



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: olvid/daemon/datatypes/v1/contact.proto
# Protobuf Python Version: 5.27.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    3,
    '',
    'olvid/daemon/datatypes/v1/contact.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .....olvid.daemon.datatypes.v1 import identity_pb2 as olvid_dot_daemon_dot_datatypes_dot_v1_dot_identity__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'olvid/daemon/datatypes/v1/contact.proto\x12\x19olvid.daemon.datatypes.v1\x1a(olvid/daemon/datatypes/v1/identity.proto\"\xe6\x02\n\x07\x43ontact\x12\x0e\n\x02id\x18\x01 \x01(\x04R\x02id\x12!\n\x0c\x64isplay_name\x18\x02 \x01(\tR\x0b\x64isplayName\x12\x44\n\x07\x64\x65tails\x18\x03 \x01(\x0b\x32*.olvid.daemon.datatypes.v1.IdentityDetailsR\x07\x64\x65tails\x12:\n\x19\x65stablished_channel_count\x18\x04 \x01(\rR\x17\x65stablishedChannelCount\x12!\n\x0c\x64\x65vice_count\x18\x05 \x01(\rR\x0b\x64\x65viceCount\x12\x38\n\x19has_one_to_one_discussion\x18\x06 \x01(\x08R\x15hasOneToOneDiscussion\x12\x1e\n\x0bhas_a_photo\x18\x07 \x01(\x08R\thasAPhoto\x12)\n\x10keycloak_managed\x18\x08 \x01(\x08R\x0fkeycloakManaged\"\xcc\x05\n\rContactFilter\x12T\n\none_to_one\x18\x01 \x01(\x0e\x32\x31.olvid.daemon.datatypes.v1.ContactFilter.OneToOneH\x00R\x08oneToOne\x88\x01\x01\x12I\n\x05photo\x18\x02 \x01(\x0e\x32..olvid.daemon.datatypes.v1.ContactFilter.PhotoH\x01R\x05photo\x88\x01\x01\x12R\n\x08keycloak\x18\x03 \x01(\x0e\x32\x31.olvid.daemon.datatypes.v1.ContactFilter.KeycloakH\x02R\x08keycloak\x88\x01\x01\x12\x33\n\x13\x64isplay_name_search\x18\x04 \x01(\tH\x03R\x11\x64isplayNameSearch\x88\x01\x01\x12V\n\x0e\x64\x65tails_search\x18\x05 \x01(\x0b\x32*.olvid.daemon.datatypes.v1.IdentityDetailsH\x04R\rdetailsSearch\x88\x01\x01\"P\n\x08OneToOne\x12\x1a\n\x16ONE_TO_ONE_UNSPECIFIED\x10\x00\x12\x11\n\rONE_TO_ONE_IS\x10\x01\x12\x15\n\x11ONE_TO_ONE_IS_NOT\x10\x02\"@\n\x05Photo\x12\x15\n\x11PHOTO_UNSPECIFIED\x10\x00\x12\r\n\tPHOTO_HAS\x10\x01\x12\x11\n\rPHOTO_HAS_NOT\x10\x02\"T\n\x08Keycloak\x12\x18\n\x14KEYCLOAK_UNSPECIFIED\x10\x00\x12\x14\n\x10KEYCLOAK_MANAGED\x10\x01\x12\x18\n\x14KEYCLOAK_NOT_MANAGED\x10\x02\x42\r\n\x0b_one_to_oneB\x08\n\x06_photoB\x0b\n\t_keycloakB\x16\n\x14_display_name_searchB\x11\n\x0f_details_searchB\xc5\x01\n\x1d\x63om.olvid.daemon.datatypes.v1B\x0c\x43ontactProtoP\x01Z\x0folvid.io/daemon\xa2\x02\x03ODD\xaa\x02\x19Olvid.Daemon.Datatypes.V1\xca\x02\x19Olvid\\Daemon\\Datatypes\\V1\xe2\x02%Olvid\\Daemon\\Datatypes\\V1\\GPBMetadata\xea\x02\x1cOlvid::Daemon::Datatypes::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'olvid.daemon.datatypes.v1.contact_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\035com.olvid.daemon.datatypes.v1B\014ContactProtoP\001Z\017olvid.io/daemon\242\002\003ODD\252\002\031Olvid.Daemon.Datatypes.V1\312\002\031Olvid\\Daemon\\Datatypes\\V1\342\002%Olvid\\Daemon\\Datatypes\\V1\\GPBMetadata\352\002\034Olvid::Daemon::Datatypes::V1'
  _globals['_CONTACT']._serialized_start=113
  _globals['_CONTACT']._serialized_end=471
  _globals['_CONTACTFILTER']._serialized_start=474
  _globals['_CONTACTFILTER']._serialized_end=1190
  _globals['_CONTACTFILTER_ONETOONE']._serialized_start=877
  _globals['_CONTACTFILTER_ONETOONE']._serialized_end=957
  _globals['_CONTACTFILTER_PHOTO']._serialized_start=959
  _globals['_CONTACTFILTER_PHOTO']._serialized_end=1023
  _globals['_CONTACTFILTER_KEYCLOAK']._serialized_start=1025
  _globals['_CONTACTFILTER_KEYCLOAK']._serialized_end=1109
# @@protoc_insertion_point(module_scope)



####
# WARNING: DO NOT EDIT: this code is automatically generated, see overlay_generator/generate_listeners_module.py
####

from __future__ import annotations
from typing import TYPE_CHECKING

if TYPE_CHECKING:
	from .. import datatypes
	from typing import Optional, Callable, Coroutine, Any, Union

from ..listeners.GenericNotificationListener import GenericNotificationListener
from .Notifications import NOTIFICATIONS


# InvitationNotificationService
# noinspection DuplicatedCode
class InvitationReceivedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Invitation], Optional[Coroutine]], checkers: list[Callable[[datatypes.Invitation], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.invitation) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.INVITATION_RECEIVED,
			handler=lambda n: handler(n.invitation),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Invitation], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.invitation))


# noinspection DuplicatedCode
class InvitationSentListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Invitation], Optional[Coroutine]], checkers: list[Callable[[datatypes.Invitation], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.invitation) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.INVITATION_SENT,
			handler=lambda n: handler(n.invitation),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Invitation], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.invitation))


# noinspection DuplicatedCode
class InvitationDeletedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Invitation], Optional[Coroutine]], checkers: list[Callable[[datatypes.Invitation], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.invitation) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.INVITATION_DELETED,
			handler=lambda n: handler(n.invitation),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Invitation], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.invitation))


# noinspection DuplicatedCode
class InvitationUpdatedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Invitation, datatypes.Invitation.Status], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Invitation, datatypes.Invitation.Status], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.invitation, n.previous_invitation_status) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.INVITATION_UPDATED,
			handler=lambda n: handler(n.invitation, n.previous_invitation_status),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Invitation, datatypes.Invitation.Status], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.invitation, n.previous_invitation_status))


# ContactNotificationService
# noinspection DuplicatedCode
class ContactNewListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Contact], Optional[Coroutine]], checkers: list[Callable[[datatypes.Contact], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.contact) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.CONTACT_NEW,
			handler=lambda n: handler(n.contact),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Contact], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.contact))


# noinspection DuplicatedCode
class ContactDeletedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Contact], Optional[Coroutine]], checkers: list[Callable[[datatypes.Contact], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.contact) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.CONTACT_DELETED,
			handler=lambda n: handler(n.contact),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Contact], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.contact))


# noinspection DuplicatedCode
class ContactDetailsUpdatedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Contact, datatypes.IdentityDetails], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Contact, datatypes.IdentityDetails], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.contact, n.previous_details) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.CONTACT_DETAILS_UPDATED,
			handler=lambda n: handler(n.contact, n.previous_details),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Contact, datatypes.IdentityDetails], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.contact, n.previous_details))


# GroupNotificationService
# noinspection DuplicatedCode
class GroupNewListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Group], Optional[Coroutine]], checkers: list[Callable[[datatypes.Group], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.group) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.GROUP_NEW,
			handler=lambda n: handler(n.group),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Group], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.group))


# noinspection DuplicatedCode
class GroupDeletedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Group], Optional[Coroutine]], checkers: list[Callable[[datatypes.Group], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.group) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.GROUP_DELETED,
			handler=lambda n: handler(n.group),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Group], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.group))


# noinspection DuplicatedCode
class GroupNameUpdatedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Group, str], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Group, str], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.group, n.previous_name) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.GROUP_NAME_UPDATED,
			handler=lambda n: handler(n.group, n.previous_name),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Group, str], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.group, n.previous_name))


# noinspection DuplicatedCode
class GroupDescriptionUpdatedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Group, str], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Group, str], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.group, n.previous_description) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.GROUP_DESCRIPTION_UPDATED,
			handler=lambda n: handler(n.group, n.previous_description),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Group, str], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.group, n.previous_description))


# noinspection DuplicatedCode
class GroupPendingMemberAddedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Group, datatypes.PendingGroupMember], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Group, datatypes.PendingGroupMember], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.group, n.pending_member) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.GROUP_PENDING_MEMBER_ADDED,
			handler=lambda n: handler(n.group, n.pending_member),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Group, datatypes.PendingGroupMember], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.group, n.pending_member))


# noinspection DuplicatedCode
class GroupPendingMemberRemovedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Group, datatypes.PendingGroupMember], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Group, datatypes.PendingGroupMember], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.group, n.pending_member) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.GROUP_PENDING_MEMBER_REMOVED,
			handler=lambda n: handler(n.group, n.pending_member),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Group, datatypes.PendingGroupMember], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.group, n.pending_member))


# noinspection DuplicatedCode
class GroupMemberJoinedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Group, datatypes.GroupMember], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Group, datatypes.GroupMember], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.group, n.member) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.GROUP_MEMBER_JOINED,
			handler=lambda n: handler(n.group, n.member),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Group, datatypes.GroupMember], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.group, n.member))


# noinspection DuplicatedCode
class GroupMemberLeftListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Group, datatypes.GroupMember], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Group, datatypes.GroupMember], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.group, n.member) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.GROUP_MEMBER_LEFT,
			handler=lambda n: handler(n.group, n.member),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Group, datatypes.GroupMember], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.group, n.member))


# noinspection DuplicatedCode
class GroupOwnPermissionsUpdatedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Group, datatypes.GroupMemberPermissions, datatypes.GroupMemberPermissions], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Group, datatypes.GroupMemberPermissions, datatypes.GroupMemberPermissions], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.group, n.permissions, n.previous_permissions) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.GROUP_OWN_PERMISSIONS_UPDATED,
			handler=lambda n: handler(n.group, n.permissions, n.previous_permissions),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Group, datatypes.GroupMemberPermissions, datatypes.GroupMemberPermissions], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.group, n.permissions, n.previous_permissions))


# noinspection DuplicatedCode
class GroupMemberPermissionsUpdatedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Group, datatypes.GroupMember, datatypes.GroupMemberPermissions], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Group, datatypes.GroupMember, datatypes.GroupMemberPermissions], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.group, n.member, n.previous_permissions) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.GROUP_MEMBER_PERMISSIONS_UPDATED,
			handler=lambda n: handler(n.group, n.member, n.previous_permissions),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Group, datatypes.GroupMember, datatypes.GroupMemberPermissions], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.group, n.member, n.previous_permissions))


# noinspection DuplicatedCode
class GroupUpdateInProgressListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[int], Optional[Coroutine]], checkers: list[Callable[[int], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.group_id) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.GROUP_UPDATE_IN_PROGRESS,
			handler=lambda n: handler(n.group_id),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[int], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.group_id))


# noinspection DuplicatedCode
class GroupUpdateFinishedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[int], Optional[Coroutine]], checkers: list[Callable[[int], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.group_id) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.GROUP_UPDATE_FINISHED,
			handler=lambda n: handler(n.group_id),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[int], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.group_id))


# DiscussionNotificationService
# noinspection DuplicatedCode
class DiscussionNewListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Discussion], Optional[Coroutine]], checkers: list[Callable[[datatypes.Discussion], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.discussion) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.DISCUSSION_NEW,
			handler=lambda n: handler(n.discussion),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Discussion], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.discussion))


# noinspection DuplicatedCode
class DiscussionLockedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Discussion], Optional[Coroutine]], checkers: list[Callable[[datatypes.Discussion], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.discussion) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.DISCUSSION_LOCKED,
			handler=lambda n: handler(n.discussion),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Discussion], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.discussion))


# noinspection DuplicatedCode
class DiscussionTitleUpdatedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Discussion, str], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Discussion, str], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.discussion, n.previous_title) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.DISCUSSION_TITLE_UPDATED,
			handler=lambda n: handler(n.discussion, n.previous_title),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Discussion, str], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.discussion, n.previous_title))


# noinspection DuplicatedCode
class DiscussionSettingsUpdatedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.DiscussionSettings, datatypes.DiscussionSettings], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.DiscussionSettings, datatypes.DiscussionSettings], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.new_settings, n.previous_settings) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.DISCUSSION_SETTINGS_UPDATED,
			handler=lambda n: handler(n.new_settings, n.previous_settings),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.DiscussionSettings, datatypes.DiscussionSettings], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.new_settings, n.previous_settings))


# MessageNotificationService
# noinspection DuplicatedCode
class MessageReceivedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Message], Optional[Coroutine]], checkers: list[Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.message) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.MESSAGE_RECEIVED,
			handler=lambda n: handler(n.message),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.message))


# noinspection DuplicatedCode
class MessageSentListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Message], Optional[Coroutine]], checkers: list[Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.message) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.MESSAGE_SENT,
			handler=lambda n: handler(n.message),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.message))


# noinspection DuplicatedCode
class MessageDeletedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Message], Optional[Coroutine]], checkers: list[Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.message) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.MESSAGE_DELETED,
			handler=lambda n: handler(n.message),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.message))


# noinspection DuplicatedCode
class MessageBodyUpdatedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Message, str], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Message, str], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.message, n.previous_body) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.MESSAGE_BODY_UPDATED,
			handler=lambda n: handler(n.message, n.previous_body),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Message, str], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.message, n.previous_body))


# noinspection DuplicatedCode
class MessageUploadedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Message], Optional[Coroutine]], checkers: list[Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.message) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.MESSAGE_UPLOADED,
			handler=lambda n: handler(n.message),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.message))


# noinspection DuplicatedCode
class MessageDeliveredListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Message], Optional[Coroutine]], checkers: list[Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.message) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.MESSAGE_DELIVERED,
			handler=lambda n: handler(n.message),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.message))


# noinspection DuplicatedCode
class MessageReadListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Message], Optional[Coroutine]], checkers: list[Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.message) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.MESSAGE_READ,
			handler=lambda n: handler(n.message),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.message))


# noinspection DuplicatedCode
class MessageLocationReceivedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Message], Optional[Coroutine]], checkers: list[Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.message) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.MESSAGE_LOCATION_RECEIVED,
			handler=lambda n: handler(n.message),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.message))


# noinspection DuplicatedCode
class MessageLocationSharingStartListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Message], Optional[Coroutine]], checkers: list[Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.message) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.MESSAGE_LOCATION_SHARING_START,
			handler=lambda n: handler(n.message),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.message))


# noinspection DuplicatedCode
class MessageLocationSharingUpdateListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Message, datatypes.MessageLocation], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Message, datatypes.MessageLocation], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.message, n.previous_location) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.MESSAGE_LOCATION_SHARING_UPDATE,
			handler=lambda n: handler(n.message, n.previous_location),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Message, datatypes.MessageLocation], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.message, n.previous_location))


# noinspection DuplicatedCode
class MessageLocationSharingEndListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Message], Optional[Coroutine]], checkers: list[Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.message) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.MESSAGE_LOCATION_SHARING_END,
			handler=lambda n: handler(n.message),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Message], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.message))


# noinspection DuplicatedCode
class MessageReactionAddedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Message, datatypes.MessageReaction], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Message, datatypes.MessageReaction], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.message, n.reaction) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.MESSAGE_REACTION_ADDED,
			handler=lambda n: handler(n.message, n.reaction),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Message, datatypes.MessageReaction], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.message, n.reaction))


# noinspection DuplicatedCode
class MessageReactionUpdatedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Message, datatypes.MessageReaction, datatypes.MessageReaction], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Message, datatypes.MessageReaction, datatypes.MessageReaction], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.message, n.reaction, n.previous_reaction) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.MESSAGE_REACTION_UPDATED,
			handler=lambda n: handler(n.message, n.reaction, n.previous_reaction),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Message, datatypes.MessageReaction, datatypes.MessageReaction], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.message, n.reaction, n.previous_reaction))


# noinspection DuplicatedCode
class MessageReactionRemovedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Message, datatypes.MessageReaction], Optional[Coroutine[Any, Any, None]]], checkers: list[Callable[[datatypes.Message, datatypes.MessageReaction], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.message, n.reaction) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.MESSAGE_REACTION_REMOVED,
			handler=lambda n: handler(n.message, n.reaction),
			checkers=checkers,
			count=count,
			priority=priority
		)

	def add_checker(self, checker: Callable[[datatypes.Message, datatypes.MessageReaction], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.message, n.reaction))


# AttachmentNotificationService
# noinspection DuplicatedCode
class AttachmentReceivedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Attachment], Optional[Coroutine]], checkers: list[Callable[[datatypes.Attachment], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.attachment) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.ATTACHMENT_RECEIVED,
			handler=lambda n: handler(n.attachment),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Attachment], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.attachment))


# noinspection DuplicatedCode
class AttachmentUploadedListener(GenericNotificationListener):
	def __init__(self, handler: Callable[[datatypes.Attachment], Optional[Coroutine]], checkers: list[Callable[[datatypes.Attachment], Union[bool, Coroutine[Any, Any, bool]]]] = None, count: int = -1, priority: int = 0):
		if checkers:
			checkers = [lambda n: checker(n.attachment) for checker in checkers]
		super().__init__(
			notification_type=NOTIFICATIONS.ATTACHMENT_UPLOADED,
			handler=lambda n: handler(n.attachment),
			checkers=checkers,
			count=count,
			priority=priority
		)
	
	def add_checker(self, checker: Callable[[datatypes.Attachment], Union[bool, Coroutine[Any, Any, bool]]]):
		super().add_checker(lambda n: checker(n.attachment))


del annotations
del TYPE_CHECKING
